"use strict";
var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var util = {};
util.INCH_TO_MILLIMETER = 25.4;
util.MILLIMETER_TO_INCH = 0.03937008;
util.FLOAT_PRECISION = 1e-3;
util.calculateFeedrate = function(feedrate, inMm) {
  return inMm === false ? feedrate : feedrate * util.MILLIMETER_TO_INCH;
};
util.nearlyEqual = function(a, b, precision) {
  var p = precision === void 0 ? util.FLOAT_PRECISION : precision;
  return Math.abs(b - a) <= p;
};
util.swapObjects = function(obj1, obj2) {
  function swapSingleField(objA, objB, key) {
    var temp;
    temp = objA[key];
    objA[key] = objB[key];
    objB[key] = temp;
  }
  __name(swapSingleField, "swapSingleField");
  var keys = Object.keys(obj1);
  var i = 0;
  for (i = 0; i < keys.length; i++) {
    if (typeof obj1[keys[i]] === "object") {
      util.swapObjects(obj1[keys[i]], obj2[keys[i]]);
    } else {
      swapSingleField(obj1, obj2, keys[i]);
    }
  }
};
util.copyObject = function(object) {
  var keys = Object.keys(object);
  var i = 0;
  var copy = {};
  for (i = 0; i < keys.length; i++) {
    if (typeof object[keys[i]] === "object") {
      copy[keys[i]] = util.copyObject(object[keys[i]]);
    } else {
      copy[keys[i]] = object[keys[i]];
    }
  }
  return copy;
};
util.movePoint = function(point, vector) {
  var keys = Object.keys(vector);
  var i = 0;
  for (i = 0; i < keys.length; i++) {
    if (point[keys[i]] !== void 0) {
      point[keys[i]] += vector[keys[i]];
    }
  }
};
util.dotProduct2 = function(v1, v2) {
  return v1.x * v2.x + v1.y * v2.y;
};
util.crossProduct2 = function(v1, v2) {
  return v1.x * v2.y - v2.x * v1.y;
};
util.lengthVector3 = function(v) {
  return Math.sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
};
util.findAxes = function(crossAxe) {
  if (crossAxe.toLowerCase() === "x") {
    return { re: "y", im: "z", cr: "x" };
  }
  if (crossAxe.toLowerCase() === "y") {
    return { re: "z", im: "x", cr: "y" };
  }
  return { re: "x", im: "y", cr: "z" };
};
util.scaleAndRotation = function(center, point, newPoint, angle, length, re, im) {
  var c = center, p = point, nP = newPoint;
  var l = length, cA = Math.cos(angle), sA = Math.sin(angle);
  var pRe = p[re], pIm = p[im], cRe = c[re], cIm = c[im];
  nP[re] = l * ((pRe - cRe) * cA - (pIm - cIm) * sA) + cRe;
  nP[im] = l * ((pIm - cIm) * cA + (pRe - cRe) * sA) + cIm;
};
util.findAngleVectors2 = function(v1, v2) {
  var sign = util.crossProduct2(v1, v2) < 0 ? -1 : 1;
  var dot = util.dotProduct2(v1, v2);
  var lV1 = Math.sqrt(v1.x * v1.x + v1.y * v1.y);
  var lV2 = Math.sqrt(v2.x * v2.x + v2.y * v2.y);
  if (lV1 === 0 || lV2 === 0) {
    return 0;
  }
  return sign * Math.acos(dot / (lV1 * lV2));
};
util.findAngleOrientedVectors2 = function(v1, v2, positive) {
  var angle = util.findAngleVectors2(v1, v2);
  if (positive === false && angle > 0) {
    return angle - Math.PI * 2;
  }
  if (positive === true && angle < 0) {
    return Math.PI * 2 + angle;
  }
  return angle;
};
util.isInclude = function(value, a, b) {
  return b < a ? b <= value && value <= a : a <= value && value <= b;
};
const util$1 = util;
var StraightLine$1 = /* @__PURE__ */ __name(function(index, start, end, parsedCommand, settings) {
  var that = this;
  that.returnLine = function() {
    return {
      lineNumber: that.index,
      type: that.word,
      start: that.start,
      end: that.end,
      feedrate: that.feedrate
    };
  };
  that.getSize = function() {
    return {
      min: {
        x: Math.min(that.start.x, that.end.x),
        y: Math.min(that.start.y, that.end.y),
        z: Math.min(that.start.z, that.end.z)
      },
      max: {
        x: Math.max(that.start.x, that.end.x),
        y: Math.max(that.start.y, that.end.y),
        z: Math.max(that.start.z, that.end.z)
      }
    };
  };
  function initialize(index2, start2, parsedCommand2, settings2) {
    that.index = index2;
    that.word = parsedCommand2.type;
    that.start = { x: start2.x, y: start2.y, z: start2.z };
    that.end = end;
    if (parsedCommand2.type === "G0") {
      that.feedrate = 0;
    } else if (parsedCommand2.f === void 0) {
      that.feedrate = settings2.feedrate;
    } else {
      that.feedrate = util$1.calculateFeedrate(parsedCommand2.f, settings2.inMm);
    }
  }
  __name(initialize, "initialize");
  initialize(index, start, parsedCommand, settings);
}, "StraightLine$1");
var CurvedLine$1 = /* @__PURE__ */ __name(function(index, start, end, parsedCommand, settings) {
  var that = this;
  function getBezierAngle() {
    var axes = util$1.findAxes(that.crossAxe);
    var cs = {
      x: that.start[axes.re] - that.center[axes.re],
      y: that.start[axes.im] - that.center[axes.im],
      z: 0
    };
    var ce = {
      x: that.end[axes.re] - that.center[axes.re],
      y: that.end[axes.im] - that.center[axes.im],
      z: 0
    };
    return util$1.findAngleOrientedVectors2(cs, ce, that.clockwise === false);
  }
  __name(getBezierAngle, "getBezierAngle");
  function getBezierRadius() {
    var axes = util$1.findAxes(that.crossAxe);
    var cs = {
      x: that.start[axes.re] - that.center[axes.re],
      y: that.start[axes.im] - that.center[axes.im],
      z: 0
    };
    return util$1.lengthVector3(cs);
  }
  __name(getBezierRadius, "getBezierRadius");
  function cubBez2DInt(angle, radius) {
    var p0 = {}, p1 = {}, p2 = {}, p3 = {};
    angle = Math.abs(angle);
    if (angle === Math.PI / 2) {
      p0 = { x: 0.707106781186548, y: 0.707106781186548, z: 0 };
      p1 = { x: 1.097631072937817, y: 0.316582489435277, z: 0 };
    } else {
      p0 = { x: Math.cos(angle / 2), y: Math.sin(angle / 2), z: 0 };
      p1 = {
        x: (4 - p0.x) / 3,
        y: (1 - p0.x) * (3 - p0.x) / (3 * p0.y),
        z: 0
      };
    }
    p0.x *= radius;
    p0.y *= radius;
    p1.x *= radius;
    p1.y *= radius;
    p2 = { x: p1.x, y: -p1.y, z: 0 };
    p3 = { x: p0.x, y: -p0.y, z: 0 };
    return { p0, p1, p2, p3 };
  }
  __name(cubBez2DInt, "cubBez2DInt");
  function cubBez2DTo3D(curve, clockwise, pitch, crossAxe) {
    var height = 0;
    if (clockwise === false) {
      util$1.swapObjects(curve.p0, curve.p3);
      util$1.swapObjects(curve.p1, curve.p2);
    }
    height = pitch / 3;
    if (crossAxe.toLowerCase() === "z") {
      curve.p0.z = 0;
      curve.p1.z = height;
      curve.p2.z = height * 2;
      curve.p3.z = height * 3;
    } else if (crossAxe.toLowerCase() === "x") {
      curve.p0.z = curve.p0.y;
      curve.p0.y = curve.p0.x;
      curve.p0.x = 0;
      curve.p1.z = curve.p1.y;
      curve.p1.y = curve.p1.x;
      curve.p1.x = height;
      curve.p2.z = curve.p2.y;
      curve.p2.y = curve.p2.x;
      curve.p2.x = height * 2;
      curve.p3.z = curve.p3.y;
      curve.p3.y = curve.p3.x;
      curve.p3.x = height * 3;
    } else if (crossAxe.toLowerCase() === "y") {
      curve.p0.z = curve.p0.x;
      curve.p0.x = curve.p0.y;
      curve.p0.y = 0;
      curve.p1.z = curve.p1.x;
      curve.p1.x = curve.p1.y;
      curve.p1.y = height;
      curve.p2.z = curve.p2.x;
      curve.p2.x = curve.p2.y;
      curve.p2.y = height * 2;
      curve.p3.z = curve.p3.x;
      curve.p3.x = curve.p3.y;
      curve.p3.y = height * 3;
    }
    return curve;
  }
  __name(cubBez2DTo3D, "cubBez2DTo3D");
  function rotAndPlaBez(curve, center, angle, re, im) {
    var c = { x: 0, y: 0, z: 0 };
    util$1.scaleAndRotation(c, curve.p0, curve.p0, angle, 1, re, im);
    util$1.scaleAndRotation(c, curve.p1, curve.p1, angle, 1, re, im);
    util$1.scaleAndRotation(c, curve.p2, curve.p2, angle, 1, re, im);
    util$1.scaleAndRotation(c, curve.p3, curve.p3, angle, 1, re, im);
    util$1.movePoint(curve.p0, center);
    util$1.movePoint(curve.p1, center);
    util$1.movePoint(curve.p2, center);
    util$1.movePoint(curve.p3, center);
  }
  __name(rotAndPlaBez, "rotAndPlaBez");
  function getFullBezier(num90, bez90, numSmall, bezSmall, pitch90) {
    var arcs = [];
    var center = util$1.copyObject(that.center);
    var axes = util$1.findAxes(that.crossAxe);
    var cs = {
      x: that.start[axes.re] - center[axes.re],
      y: that.start[axes.im] - center[axes.im]
    };
    var i = 0, angle = 0, sign = that.clockwise === true ? -1 : 1;
    if (num90 === 0 && numSmall === 0) {
      return arcs;
    }
    if (num90 > 0) {
      angle = util$1.findAngleOrientedVectors2({ x: bez90.p0[axes.re], y: bez90.p0[axes.im] }, cs, that.clockwise === false);
      for (i = 0; i < num90; i++) {
        arcs.push(util$1.copyObject(bez90));
        rotAndPlaBez(arcs[i], center, angle, axes.re, axes.im);
        angle += 1.570796326794897 * sign;
        center[that.crossAxe] += pitch90;
      }
    }
    if (numSmall > 0) {
      angle = util$1.findAngleOrientedVectors2({ x: bezSmall.p0[axes.re], y: bezSmall.p0[axes.im] }, cs, that.clockwise === false);
      if (num90 !== 0) {
        angle += num90 * 1.570796326794897 * sign;
      }
      arcs.push(util$1.copyObject(bezSmall));
      rotAndPlaBez(arcs[i], center, angle, axes.re, axes.im);
    }
    arcs[0].p0.x = that.start.x;
    arcs[0].p0.y = that.start.y;
    arcs[0].p0.z = that.start.z;
    arcs[arcs.length - 1].p3.x = that.end.x;
    arcs[arcs.length - 1].p3.y = that.end.y;
    arcs[arcs.length - 1].p3.z = that.end.z;
    return arcs;
  }
  __name(getFullBezier, "getFullBezier");
  function arcToBezier() {
    var num90 = 0, numSmall = 1;
    var bez90 = {}, bezSmall = {};
    var p90 = 0, pLittle = 0, pAngle = 0;
    var angle = getBezierAngle();
    var radius = getBezierRadius();
    var absAngle = Math.abs(angle), halfPI = 1.570796326794897;
    if (angle === 0 || radius === 0) {
      return [];
    }
    if (absAngle > halfPI) {
      num90 = parseInt(absAngle / halfPI, 10);
      numSmall = absAngle % halfPI !== 0 ? 1 : 0;
    }
    pAngle = (that.end[that.crossAxe] - that.start[that.crossAxe]) / absAngle;
    p90 = halfPI * pAngle;
    pLittle = (absAngle - num90 * halfPI) * pAngle;
    if (num90 > 0) {
      bez90 = cubBez2DInt(halfPI, radius);
      cubBez2DTo3D(bez90, angle < 0, p90, that.crossAxe);
    }
    if (numSmall > 0) {
      angle = absAngle - num90 * halfPI;
      if (that.clockwise === true) {
        angle = -angle;
      }
      bezSmall = cubBez2DInt(angle, radius);
      cubBez2DTo3D(bezSmall, angle < 0, pLittle, that.crossAxe);
    }
    return getFullBezier(num90, bez90, numSmall, bezSmall, p90);
  }
  __name(arcToBezier, "arcToBezier");
  function circleToBezier() {
    var bez90 = {};
    var bezier = [];
    var pitch = 0;
    var halfPI = 1.570796326794897;
    var sign = that.clockwise === true ? -1 : 1;
    var rotAngle = sign * Math.PI * 2;
    var radius = getBezierRadius();
    var i = 0;
    var center = util$1.copyObject(that.center);
    var axes = util$1.findAxes(that.crossAxe);
    if (radius === 0) {
      return [];
    }
    pitch = (that.end[that.crossAxe] - that.start[that.crossAxe]) / 4;
    bez90 = cubBez2DInt(halfPI, radius);
    cubBez2DTo3D(bez90, that.clockwise, pitch, that.crossAxe);
    for (i = 0; i < 4; i++) {
      bezier.push(util$1.copyObject(bez90));
      rotAndPlaBez(bezier[i], center, rotAngle, axes.re, axes.im);
      rotAngle += halfPI * sign;
      center[that.crossAxe] += pitch;
    }
    return getFullBezier(4, bez90, 0, bez90, pitch);
  }
  __name(circleToBezier, "circleToBezier");
  that.returnLine = function() {
    var bez = [];
    var axes = util$1.findAxes(that.crossAxe);
    if (that.start[axes.re] === that.end[axes.re] && that.start[axes.im] === that.end[axes.im]) {
      bez = circleToBezier();
    } else {
      bez = arcToBezier();
    }
    if (bez.length === 0) {
      return false;
    }
    return {
      lineNumber: that.index,
      type: that.word,
      beziers: bez,
      feedrate: that.feedrate
    };
  };
  function findCenterWithRadius(start2, end2, radius, clockwise, crossAxe) {
    var se = { x: end2.x - start2.x, y: end2.y - start2.y, z: end2.z - start2.z };
    var angle = 0, l = 1, lSE = 0, r = Math.abs(radius), aCSCE = 0;
    var center = { x: 0, y: 0, z: 0 };
    var axes = util$1.findAxes(crossAxe);
    lSE = Math.sqrt(se[axes.re] * se[axes.re] + se[axes.im] * se[axes.im]);
    if (lSE > Math.abs(radius * 2) || lSE === 0) {
      return false;
    }
    angle = Math.acos(lSE / (2 * r));
    l = r / lSE;
    util$1.scaleAndRotation(start2, end2, center, angle, l, axes.re, axes.im);
    aCSCE = util$1.findAngleVectors2({
      x: start2[axes.re] - center[axes.re],
      y: start2[axes.im] - center[axes.im]
    }, { x: end2[axes.re] - center[axes.re], y: end2[axes.im] - center[axes.im] });
    if (clockwise === true) {
      if (radius > 0 && -Math.PI <= aCSCE && aCSCE <= 0) {
        return center;
      }
      if (radius < 0 && 0 <= aCSCE && aCSCE <= Math.PI) {
        return center;
      }
    } else {
      if (radius > 0 && 0 <= aCSCE && aCSCE <= Math.PI) {
        return center;
      }
      if (radius < 0 && -Math.PI <= aCSCE && aCSCE <= 0) {
        return center;
      }
    }
    util$1.scaleAndRotation(start2, end2, center, -angle, l, axes.re, axes.im);
    return center;
  }
  __name(findCenterWithRadius, "findCenterWithRadius");
  function findCenter(start2, end2, parsedCommand2, clockwise, crossAxe, inMm) {
    var delta = inMm === false ? 1 : util$1.MILLIMETER_TO_INCH;
    var zeroPrecision = inMm ? 1e-5 : util$1.FLOAT_PRECISION;
    var comparePrecision = inMm ? 1e-4 : util$1.FLOAT_PRECISION;
    var center = { x: start2.x, y: start2.y, z: start2.z };
    var distCenterStart, distCenterEnd;
    var axes = util$1.findAxes(crossAxe);
    if (parsedCommand2.r === void 0) {
      if (parsedCommand2.i !== void 0) {
        center.x += parsedCommand2.i * delta;
      }
      if (parsedCommand2.j !== void 0) {
        center.y += parsedCommand2.j * delta;
      }
      if (parsedCommand2.k !== void 0) {
        center.z += parsedCommand2.k * delta;
      }
      distCenterStart = Math.pow(center[axes.re] - start2[axes.re], 2);
      distCenterStart += Math.pow(center[axes.im] - start2[axes.im], 2);
      distCenterEnd = Math.pow(center[axes.re] - end2[axes.re], 2);
      distCenterEnd += Math.pow(center[axes.im] - end2[axes.im], 2);
      if (util$1.nearlyEqual(distCenterStart, 0, zeroPrecision) === true || util$1.nearlyEqual(distCenterEnd, 0, zeroPrecision) === true) {
        return false;
      }
      if (util$1.nearlyEqual(distCenterStart, distCenterEnd, comparePrecision) === false) {
        return false;
      }
    } else {
      center = findCenterWithRadius(start2, end2, parsedCommand2.r * delta, clockwise, crossAxe);
      if (center === false) {
        return false;
      }
    }
    center[crossAxe] = start2[crossAxe];
    return center;
  }
  __name(findCenter, "findCenter");
  function axeCutArc(reValue, imValue, angleBezier, cs) {
    var a = util$1.findAngleOrientedVectors2(cs, { x: reValue, y: imValue }, that.clockwise === false);
    return util$1.isInclude(a, 0, angleBezier) === true;
  }
  __name(axeCutArc, "axeCutArc");
  that.getSize = function() {
    var axes = util$1.findAxes(that.crossAxe);
    var cs = {
      x: that.start[axes.re] - that.center[axes.re],
      y: that.start[axes.im] - that.center[axes.im]
    };
    var radius = getBezierRadius(), aBez = getBezierAngle();
    var min = { x: 0, y: 0, z: 0 }, max = { x: 0, y: 0, z: 0 };
    if (that.start[axes.re] === that.end[axes.re] && that.start[axes.im] === that.end[axes.im]) {
      min[axes.re] = that.center[axes.re] - radius;
      min[axes.im] = that.center[axes.im] - radius;
      min[axes.cr] = Math.min(that.start[axes.cr], that.end[axes.cr]);
      max[axes.re] = that.center[axes.re] + radius;
      max[axes.im] = that.center[axes.im] + radius;
      max[axes.cr] = Math.max(that.start[axes.cr], that.end[axes.cr]);
      return { min, max };
    }
    min.x = Math.min(that.start.x, that.end.x);
    min.y = Math.min(that.start.y, that.end.y);
    min.z = Math.min(that.start.z, that.end.z);
    max.x = Math.max(that.start.x, that.end.x);
    max.y = Math.max(that.start.y, that.end.y);
    max.z = Math.max(that.start.z, that.end.z);
    if (axeCutArc(0, 1, aBez, cs) === true) {
      max[axes.im] = that.center[axes.im] + radius;
    }
    if (axeCutArc(0, -1, aBez, cs) === true) {
      min[axes.im] = that.center[axes.im] - radius;
    }
    if (axeCutArc(1, 0, aBez, cs) === true) {
      max[axes.re] = that.center[axes.re] + radius;
    }
    if (axeCutArc(-1, 0, aBez, cs) === true) {
      min[axes.re] = that.center[axes.re] - radius;
    }
    return { min, max };
  };
  function initialize(index2, start2, parsedCommand2, settings2) {
    that.index = index2;
    that.word = parsedCommand2.type;
    that.start = { x: start2.x, y: start2.y, z: start2.z };
    that.end = end;
    that.clockwise = parsedCommand2.type === "G2";
    that.center = findCenter(start2, that.end, parsedCommand2, that.clockwise, settings2.crossAxe, settings2.inMm);
    that.crossAxe = settings2.crossAxe;
    if (parsedCommand2.f === void 0) {
      that.feedrate = settings2.feedrate;
    } else {
      that.feedrate = util$1.calculateFeedrate(parsedCommand2.f, settings2.inMm);
    }
  }
  __name(initialize, "initialize");
  initialize(index, start, parsedCommand, settings);
}, "CurvedLine$1");
const parseLine = (() => {
  const re = /(%.*)|({.*)|((?:\$\$)|(?:\$[a-zA-Z0-9#]*))|([a-zA-Z][0-9\+\-\.]+)|(\*[0-9]+)/gim;
  return (line, options) => {
    const result = {
      line,
      words: []
    };
    let ln;
    const words = line.match(re) || [];
    for (let i = 0, l = words.length; i < l; ++i) {
      const word = words[i];
      const letter = word[0];
      const argument = word.slice(1);
      if (letter === "%") {
        result.cmds = (result.cmds || []).concat(line.trim());
        continue;
      }
      if (letter === "{") {
        result.cmds = (result.cmds || []).concat(line.trim());
        continue;
      }
      if (letter === "$") {
        result.cmds = (result.cmds || []).concat(`${letter}${argument}`);
        continue;
      }
      if (letter === "N" && typeof ln === "undefined") {
        ln = Number(argument);
        continue;
      }
      let value = Number(argument);
      if (Number.isNaN(value)) {
        value = argument;
      }
      result.words.push([letter, value]);
    }
    typeof ln !== "undefined" && (result.ln = ln);
    return result;
  };
})();
var StraightLine = { StraightLine: StraightLine$1 }.StraightLine;
var CurvedLine = { CurvedLine: CurvedLine$1 }.CurvedLine;
var parse = /* @__PURE__ */ __name(function(code) {
  var unitIsSet = false;
  var setInInch = true;
  function bulkRemoveCommentsAndSpaces(gcode2) {
    return gcode2.replace(/[ \r]|[(;%].*/g, "");
  }
  __name(bulkRemoveCommentsAndSpaces, "bulkRemoveCommentsAndSpaces");
  function parseParsedGCode(parsed) {
    var obj = {};
    var i2 = 0;
    var l;
    var letter = "", number = "";
    var tab = [];
    var emptyObj = true;
    for (i2 = 0, l = parsed.words.length; i2 < l; i2++) {
      letter = parsed.words[i2][0];
      number = parsed.words[i2][1];
      if (letter === "G" || letter === "M") {
        if (emptyObj === false) {
          tab.push(obj);
          obj = {};
        }
        obj.type = letter + number;
        emptyObj = false;
      } else {
        obj[letter.toLowerCase()] = parseFloat(number, 10);
      }
    }
    tab.push(obj);
    return tab;
  }
  __name(parseParsedGCode, "parseParsedGCode");
  function checkTotalSize(totalSize2, size) {
    var keys = ["x", "y", "z"];
    var i2 = 0;
    for (i2 = keys.length - 1; i2 >= 0; i2--) {
      if (totalSize2.min[keys[i2]] > size.min[keys[i2]]) {
        totalSize2.min[keys[i2]] = size.min[keys[i2]];
      }
      if (totalSize2.max[keys[i2]] < size.max[keys[i2]]) {
        totalSize2.max[keys[i2]] = size.max[keys[i2]];
      }
    }
  }
  __name(checkTotalSize, "checkTotalSize");
  function setGoodType(parsedCommand, previousMoveCommand) {
    if (parsedCommand.type !== void 0) {
      return;
    }
    if (previousMoveCommand !== "") {
      parsedCommand.type = previousMoveCommand;
    }
  }
  __name(setGoodType, "setGoodType");
  function findPosition(start, parameters, relative, inMm) {
    var pos = { x: start.x, y: start.y, z: start.z };
    var d = inMm === false ? 1 : util$1.MILLIMETER_TO_INCH;
    if (relative === true) {
      if (parameters.x !== void 0) {
        pos.x += parameters.x * d;
      }
      if (parameters.y !== void 0) {
        pos.y += parameters.y * d;
      }
      if (parameters.z !== void 0) {
        pos.z += parameters.z * d;
      }
    } else {
      if (parameters.x !== void 0) {
        pos.x = parameters.x * d;
      }
      if (parameters.y !== void 0) {
        pos.y = parameters.y * d;
      }
      if (parameters.z !== void 0) {
        pos.z = parameters.z * d;
      }
    }
    return pos;
  }
  __name(findPosition, "findPosition");
  function checkG2G3(command, errorList2, line, previousFeedrate) {
    if (command.r === void 0 && command.i === void 0 && command.j === void 0 && command.k === void 0) {
      return false;
    }
    if (command.r !== void 0 && (command.i !== void 0 || command.j !== void 0 || command.k !== void 0)) {
      return false;
    }
    return true;
  }
  __name(checkG2G3, "checkG2G3");
  function manageG0G1(command, settings2, lineNumber, lines2, totalSize2) {
    var nextPosition = findPosition(settings2.position, command, settings2.relative, settings2.inMm);
    var line = new StraightLine(lineNumber, settings2.position, nextPosition, command, settings2);
    settings2.previousMoveCommand = command.type;
    checkTotalSize(totalSize2, line.getSize());
    lines2.push(line.returnLine());
    settings2.position = util$1.copyObject(line.end);
  }
  __name(manageG0G1, "manageG0G1");
  function manageG2G3(command, settings2, lineNumber, lines2, totalSize2, errorList2) {
    var nextPosition = findPosition(settings2.position, command, settings2.relative, settings2.inMm);
    var line = new CurvedLine(lineNumber, settings2.position, nextPosition, command, settings2);
    if (line.center !== false) {
      var temp = line.returnLine();
      if (temp === false) {
        return;
      }
      settings2.previousMoveCommand = command.type;
      checkTotalSize(totalSize2, line.getSize());
      lines2.push(temp);
      settings2.position = util$1.copyObject(line.end);
    }
  }
  __name(manageG2G3, "manageG2G3");
  function manageCommand(command, settings2, lineNumber, lines2, totalSize2, errorList2) {
    if (command.type === void 0 && Object.keys(command).length === 0) {
      return true;
    }
    setGoodType(command, settings2.previousMoveCommand);
    if (command.type === void 0)
      ;
    else if (command.type === "G0") {
      manageG0G1(command, settings2, lineNumber, lines2, totalSize2);
    } else if (command.type === "G1") {
      manageG0G1(command, settings2, lineNumber, lines2, totalSize2);
    } else if ((command.type === "G2" || command.type === "G3") && checkG2G3(command) === true) {
      manageG2G3(command, settings2, lineNumber, lines2, totalSize2);
    } else if (command.type === "G17") {
      settings2.crossAxe = "z";
    } else if (command.type === "G18") {
      settings2.crossAxe = "y";
    } else if (command.type === "G19") {
      settings2.crossAxe = "x";
    } else if (command.type === "G20") {
      settings2.inMm = false;
      if (unitIsSet === false) {
        setInInch = true;
        unitIsSet = true;
      }
    } else if (command.type === "G21") {
      settings2.inMm = true;
      if (unitIsSet === false) {
        setInInch = false;
        unitIsSet = true;
      }
    } else if (command.type === "G90") {
      settings2.relative = false;
    } else if (command.type === "G91") {
      settings2.relative = true;
    } else if (command.type === "M2") {
      return false;
    }
    return true;
  }
  __name(manageCommand, "manageCommand");
  var totalSize = {
    min: { x: 0, y: 0, z: 0 },
    max: { x: 0, y: 0, z: 0 }
  };
  var i = 0, j = 0;
  var tabRes = [];
  var parsing = true;
  var lines = [];
  var errorList = [];
  var settings = {
    previousMoveCommand: "",
    crossAxe: "z",
    inMm: false,
    relative: false,
    position: { x: 0, y: 0, z: 0 }
  };
  if (typeof code !== "string" || code === "") {
    return {
      gcode: [],
      lines: [],
      size: totalSize,
      displayInInch: setInInch,
      errorList: [{ line: 0, message: "(error) No command." }]
    };
  }
  var gcode = bulkRemoveCommentsAndSpaces(code).toUpperCase().split("\n");
  var len = gcode.length;
  while (i < len && parsing === true) {
    const parsed = parseLine(gcode[i]);
    tabRes = parseParsedGCode(parsed);
    j = 0;
    const tabLen = tabRes.length;
    while (j < tabLen && parsing === true) {
      parsing = manageCommand(tabRes[j], settings, i + 1, lines, totalSize);
      j++;
    }
    i++;
  }
  return {
    lines,
    size: totalSize,
    displayInInch: setInInch,
    errorList
  };
}, "parse");
exports.parse = parse;
