#!/usr/bin/env node
"use strict";
var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
const vue = require("vue");
const pinia = require("pinia");
const SparkMD5 = require("spark-md5");
const mitt = require("mitt");
const io = require("socket.io-client");
const jwt = require("jsonwebtoken");
const program = require("commander");
const fs$1 = require("fs");
const path$1 = require("path");
const node = require("@elgato-stream-deck/node");
const tinycolor = require("tinycolor2");
const evaluate = require("simple-evaluate");
const sharp = require("sharp");
const escape = require("lodash/escape");
const throttle = require("lodash/throttle");
const _interopDefaultLegacy = /* @__PURE__ */ __name((e) => e && typeof e === "object" && "default" in e ? e : { default: e }, "_interopDefaultLegacy");
const SparkMD5__default = /* @__PURE__ */ _interopDefaultLegacy(SparkMD5);
const mitt__default = /* @__PURE__ */ _interopDefaultLegacy(mitt);
const io__default = /* @__PURE__ */ _interopDefaultLegacy(io);
const jwt__default = /* @__PURE__ */ _interopDefaultLegacy(jwt);
const program__default = /* @__PURE__ */ _interopDefaultLegacy(program);
const fs__default = /* @__PURE__ */ _interopDefaultLegacy(fs$1);
const path__default = /* @__PURE__ */ _interopDefaultLegacy(path$1);
const tinycolor__default = /* @__PURE__ */ _interopDefaultLegacy(tinycolor);
const evaluate__default = /* @__PURE__ */ _interopDefaultLegacy(evaluate);
const sharp__default = /* @__PURE__ */ _interopDefaultLegacy(sharp);
const escape__default = /* @__PURE__ */ _interopDefaultLegacy(escape);
const throttle__default = /* @__PURE__ */ _interopDefaultLegacy(throttle);
const animation = /* @__PURE__ */ __name(({ duration, fps, loop = false, callback }) => {
  let timeout;
  let startTime;
  const delayTime = 1e3 / fps;
  const max = loop ? Infinity : 1;
  const percentCallback = /* @__PURE__ */ __name(() => {
    const percent = Math.max(0, Math.min(max, (Date.now() - startTime) / duration));
    callback(percent % (loop ? 1 : Infinity));
    if (percent < 1 || loop) {
      run2();
    }
  }, "percentCallback");
  const run2 = /* @__PURE__ */ __name(() => {
    timeout = setTimeout(percentCallback, delayTime);
  }, "run");
  const cancel = /* @__PURE__ */ __name(() => {
    clearTimeout(timeout);
  }, "cancel");
  const start = /* @__PURE__ */ __name(() => {
    cancel();
    startTime = Date.now();
    run2();
  }, "start");
  const delay = /* @__PURE__ */ __name((when) => {
    timeout = setTimeout(start, when - delayTime);
  }, "delay");
  return {
    start,
    delay,
    cancel
  };
}, "animation");
const Sleep = /* @__PURE__ */ __name((delay) => {
  let timeout;
  const asleep = vue.ref(false);
  const beginTimeout = /* @__PURE__ */ __name(() => {
    if (!delay) {
      return;
    }
    timeout = setTimeout(() => {
      asleep.value = true;
    }, delay * 1e3);
  }, "beginTimeout");
  const clearSleep = /* @__PURE__ */ __name(() => {
    if (asleep.value) {
      asleep.value = false;
    }
    clearTimeout(timeout);
    beginTimeout();
  }, "clearSleep");
  beginTimeout();
  return {
    clearSleep,
    asleep
  };
}, "Sleep");
const useUiStore = pinia.defineStore({
  id: "ui",
  state: () => ({
    active: true,
    activityTimeout: null,
    asleep: false,
    bgColor: 2,
    brightness: 60,
    columns: 5,
    font: "monospace",
    fontSize: 12,
    lineHeight: 1.1,
    feedrateInterval: 1,
    spindleInterval: 1,
    fileDetailsPath: null,
    fileDetails: {},
    fileDetailsSort: "alpha_asc",
    gcodeColors: {},
    gcodeLimit: 0,
    iconSize: 72,
    input: {
      value: "",
      previous: 0,
      type: "",
      callback: () => {
      }
    },
    pageColor: null,
    palette: ["#000", "#fff"],
    progressColor: 4,
    rows: 3,
    sceneStack: [],
    textColor: 1,
    textShadow: false,
    throttle: 0,
    timeout: 0,
    userFlags: {},
    web: true
  }),
  getters: {
    sceneName: (state) => {
      return state.sceneStack[state.sceneStack.length - 1];
    },
    fileDetailSize: (state) => {
      if (!state.fileDetails) {
        return "0";
      }
      const size = state.fileDetails.size;
      if (size > 1024) {
        const digits = size > 5120 ? 0 : 1;
        return `${+(size / 1024).toFixed(digits)}K`;
      } else {
        return `${size}`;
      }
    },
    fileDetailModifiedTime: (state) => {
      if (!state.fileDetails) {
        return "";
      }
      return new Date(state.fileDetails.mtime).toLocaleString();
    },
    fileDetailCreatedTime: (state) => {
      if (!state.fileDetails) {
        return "";
      }
      return new Date(state.fileDetails.ctime).toLocaleString();
    },
    isWeb: () => false,
    displayBrightness: (state) => state.brightness
  },
  actions: {
    addInput(chars) {
      const inputTest = /^-?(0|[1-9]\d*)?(?:\.\d*)?$/;
      const newValue = (this.input.value + chars).replace(/^0+(?=[1-9]|0)/, "");
      if (inputTest.test(newValue)) {
        this.input.value = newValue;
      }
    },
    toggleInputSign() {
      if (this.input.value.startsWith("-")) {
        this.input.value = this.input.value.slice(1);
      } else {
        this.input.value = "-" + this.input.value;
      }
    },
    startInput(value, type, scene = "numpad", callback = () => {
    }) {
      this.input.value = "";
      this.input.previous = value;
      this.input.type = type;
      this.input.callback = callback;
      this.goToScene(scene);
    },
    completeInput() {
      this.input.callback(this.input.value);
      this.input.value = "";
      this.goBack();
    },
    toggleFeedrateInterval() {
      this.feedrateInterval = this.feedrateInterval === 1 ? 10 : 1;
    },
    toggleSpindleInterval() {
      this.spindleInterval = this.spindleInterval === 1 ? 10 : 1;
    },
    setGrid(rows, columns) {
      this.columns = columns;
      this.rows = rows;
    },
    setBgColor(color) {
      this.bgColor = color;
    },
    clearUserFlag(id) {
      delete this.userFlags[id];
    },
    setBrightness(brightness) {
      if (brightness != null) {
        this.brightness = Math.max(Math.min(100, brightness), 10);
      }
    },
    setIconSize(size) {
      this.iconSize = size;
    },
    setGcodeColors(colors) {
      if (!colors) {
        return;
      }
      this.gcodeColors = Object.freeze(colors);
    },
    activity() {
      this.active = true;
      clearTimeout(this.activityTimeout);
      this.activityTimeout = setTimeout(this.inactive.bind(this), this.timeout);
    },
    inactive() {
      this.active = false;
    },
    decreaseBrightness() {
      this.setBrightness(this.brightness - 10);
    },
    increaseBrightness() {
      this.setBrightness(this.brightness + 10);
    },
    setGcodeLimit(limit) {
      this.gcodeLimit = limit;
    },
    setTimeout(timeout) {
      this.timeout = timeout;
    },
    setWeb(web) {
      this.web = !!web;
    },
    setUserFlag(id, value) {
      this.userFlags[id] = value;
    },
    setProgressColor(color) {
      this.progressColor = color;
    },
    setPalette(colors) {
      this.palette = colors;
    },
    setThrottle(throttle2) {
      if (throttle2 != null) {
        const millis = parseInt(throttle2);
        if (millis) {
          this.throttle = millis;
        }
      }
    },
    setScene(scene) {
      this.sceneStack.splice(0);
      this.sceneStack.push(scene);
    },
    swapScene(scene) {
      this.sceneStack.splice(this.sceneStack.length - 1, 1, scene);
    },
    goToScene(scene) {
      if (this.sceneName !== scene) {
        this.sceneStack.push(scene);
      }
    },
    goBack(count = 1) {
      if (this.sceneStack.length > 1) {
        this.sceneStack.splice(-count, count);
      }
    }
  }
});
const SleepScreen = /* @__PURE__ */ __name((timeout, streamdeck) => {
  const ui = useUiStore();
  const sleep = Sleep(timeout);
  let fadeoutAnimation;
  vue.watch(sleep.asleep, (asleep) => {
    ui.asleep = asleep;
    fadeoutAnimation == null ? void 0 : fadeoutAnimation.cancel();
    fadeoutAnimation = null;
    if (asleep) {
      fadeoutAnimation = animation({
        duration: 800 * ui.displayBrightness * 0.01,
        fps: 30,
        callback: (percent) => {
          streamdeck.setBrightness(Math.floor(ui.displayBrightness * (1 - percent)));
        }
      });
      fadeoutAnimation.start();
    } else {
      streamdeck.setBrightness(ui.displayBrightness);
    }
  });
  const wake = /* @__PURE__ */ __name(() => {
    const wasAsleep = ui.asleep;
    sleep.clearSleep();
    return wasAsleep;
  }, "wake");
  return { wake };
}, "SleepScreen");
const useButtonStore = pinia.defineStore({
  id: "buttons",
  state: () => ({
    _buttons: {}
  }),
  getters: {
    buttons: (state) => state._buttons
  },
  actions: {
    setButtons(buttons) {
      this._buttons = buttons;
    }
  }
});
const jogDistances = {
  mm: [1e-3, 0.01, 0.1, 1, 10, 20, 50, 100],
  in: [1e-3, 0.01, 0.1, 1, 5, 10, 20]
};
const jogSpeeds = {
  mm: [500, 2500, 5e3],
  in: [15, 75, 150]
};
const changeListPosition = /* @__PURE__ */ __name((item, list, fallback, offset = 1) => {
  const index = list.indexOf(item);
  if (index === -1) {
    return fallback;
  }
  return list[Math.min(list.length - 1, Math.max(0, index + offset))];
}, "changeListPosition");
const listIncrease = /* @__PURE__ */ __name((item, list, fallback) => changeListPosition(item, list, fallback, 1), "listIncrease");
const listDecrease = /* @__PURE__ */ __name((item, list, fallback) => changeListPosition(item, list, fallback, -1), "listDecrease");
const axisOrder = ["x", "y", "z", "a", "b", "c"];
const cncStates = {
  IDLE: "Idle",
  HOLD: "Hold",
  ALARM: "Alarm",
  JOG: "Jog",
  RUNNING: "Run",
  HOMING: "Home"
};
const workflowStates = {
  IDLE: "idle",
  PAUSE: "paused",
  RUNNING: "running"
};
const feederStates = {
  IDLE: "idle",
  PAUSE: "paused"
};
const formatDuration = /* @__PURE__ */ __name((seconds) => {
  return new Date(seconds * 1e3).toISOString().substring(11, 19);
}, "formatDuration");
const formatAxes = /* @__PURE__ */ __name((position, axes) => axes.map((axis) => {
  return `${axis.toUpperCase()}:${position[axis].padStart(8, "\xA0")}`;
}).join("\n"), "formatAxes");
const pauseText = /* @__PURE__ */ __name((reason, message) => {
  if (reason === "M0" || reason === "M1") {
    return message;
  }
  if (reason !== "M6") {
    return;
  }
  const messages = ["Tool change"];
  if (message && message !== "M6") {
    messages.push(message.replace(/^M6 \((.*)\)$/, "$1"));
  }
  return messages.join("\n");
}, "pauseText");
const useCncStore = pinia.defineStore({
  id: "cnc",
  state: () => ({
    connected: false,
    connecting: false,
    token: null,
    runState: cncStates.IDLE,
    workflowState: workflowStates.IDLE,
    feederState: feederStates.IDLE,
    locked: false,
    client: null,
    macros: null,
    commands: null,
    elapsedTime: null,
    remainingTime: null,
    alarmReason: "",
    pauseReason: "",
    pauseMessage: "",
    errorMessage: "",
    feedPauseReason: "",
    feedPauseMessage: "",
    jogDistance: 1,
    jogSpeed: 500,
    settings: {},
    overrides: {
      feed: 100,
      spindle: 100,
      rapid: 100
    },
    wpos: {
      x: "0.000",
      y: "0.000",
      z: "0.000",
      a: "0.000",
      b: "0.000",
      c: "0.000"
    },
    mpos: {
      x: "0.000",
      y: "0.000",
      z: "0.000",
      a: "0.000",
      b: "0.000",
      c: "0.000"
    },
    modal: {
      distance: "G90",
      units: "G21",
      wcs: "G54"
    },
    axes: ["x", "y", "z", "a", "b", "c"],
    spindleRpm: 0,
    feedRate: 0,
    activeCommands: {},
    activeWorkers: {}
  }),
  actions: {
    setClient(client) {
      this.client = client;
    },
    setConnected(connected) {
      this.connected = connected;
      this.connecting = false;
    },
    setConnecting(connecting = true) {
      this.connecting = connecting;
    },
    setToken(token) {
      this.token = token;
    },
    setElapsedTime(time) {
      this.elapsedTime = time;
    },
    setRemainingTime(time) {
      this.remainingTime = time;
    },
    setRunState(state) {
      if (Object.values(cncStates).includes(state)) {
        this.runState = state;
        if (state !== cncStates.ALARM) {
          this.locked = false;
        }
      } else if (state) {
        console.error("Unrecognized run state", state);
      }
    },
    setWorkflowState(state) {
      if (Object.values(workflowStates).includes(state)) {
        this.workflowState = state;
        if (state !== workflowStates.PAUSE) {
          this.clearPause();
          this.clearError();
        }
      } else if (state) {
        console.error("Unrecognized workflow state", state);
      }
    },
    setFeedHold(data, message) {
      this.feederState = feederStates.PAUSE;
      this.feedPauseReason = data;
      this.feedPauseMessage = message;
    },
    clearFeedHold() {
      this.feederState = feederStates.IDLE;
      this.feedPauseReason = null;
      this.feedPauseMessage = null;
    },
    setAxes(axes) {
      if (axes) {
        this.axes = Object.freeze(axes);
      }
    },
    setVersion(version) {
      this.version = version;
    },
    setSettings(settings) {
      this.settings = settings;
    },
    increaseJogDistance() {
      this.jogDistance = listIncrease(this.jogDistance, this.distances, 1);
    },
    decreaseJogDistance() {
      this.jogDistance = listDecrease(this.jogDistance, this.distances, 1);
    },
    increaseJogSpeed() {
      this.jogSpeed = listIncrease(this.jogSpeed, this.speeds, this.speedFallback);
    },
    decreaseJogSpeed() {
      this.jogSpeed = listDecrease(this.jogSpeed, this.speeds, this.speedFallback);
    },
    setError(error) {
      this.errorMessage = error;
    },
    clearError(error) {
      this.errorMessage = null;
    },
    setPause(reason, message) {
      this.workflowState = workflowStates.PAUSE;
      this.pauseReason = reason;
      this.pauseMessage = message;
    },
    clearPause() {
      this.pauseReason = null;
      this.pauseMessage = null;
    },
    setAlarm(reason = "") {
      this.runState = cncStates.ALARM;
      this.alarmReason = reason;
    },
    setLocked(val = true) {
      this.locked = val;
    },
    setSpindleRpm(val = 0) {
      this.spindleRpm = parseInt(val);
    },
    setFeedrate(val = 0) {
      this.feedRate = parseInt(val);
    },
    setModal(modal) {
      Object.entries(modal).forEach(([key, value]) => {
        this.modal[key] = value;
      });
      if (this.modal.units !== modal.units) {
        this.jogDistance = 1;
        this.jogSpeed = this.speedFallback;
      }
    },
    setMpos(mpos) {
      Object.entries(mpos).forEach(([key, value]) => {
        this.mpos[key] = value;
      });
    },
    setWpos(wpos) {
      Object.entries(wpos).forEach(([key, value]) => {
        this.wpos[key] = value;
      });
    },
    setOverrides(feed, rapid, spindle) {
      this.overrides = {
        feed,
        rapid,
        spindle
      };
    },
    async loadMacros() {
      if (!this.client) {
        return;
      }
      const macros = await this.client.fetch("macros");
      if (!macros) {
        return;
      }
      this.macros = macros.records.reduce((lookup, macro) => {
        lookup[macro.name] = macro.id;
        return lookup;
      }, {});
    },
    addActiveCommand(commandId, taskId) {
      this.activeCommands[commandId] = taskId;
    },
    clearActiveCommand(taskId) {
      Object.keys(this.activeCommands).forEach((key) => {
        if (this.activeCommands[key] === taskId) {
          delete this.activeCommands[key];
        }
      });
    },
    clearActiveCommands() {
      Object.keys(this.activeCommands).forEach((key) => {
        delete this.activeCommands[key];
      });
    },
    commandRunning(id) {
      return !!this.activeCommands[id];
    },
    addActiveWorker(workerId) {
      this.activeWorkers[workerId] = true;
    },
    clearActiveWorker(workerId) {
      delete this.activeWorkers[workerId];
    },
    workerRunning(workerId) {
      return !!this.activeWorkers[workerId];
    },
    async loadCommands() {
      if (!this.client) {
        return;
      }
      const commands = await this.client.fetch("commands");
      if (!commands) {
        return;
      }
      this.commands = commands.records.reduce((lookup, command) => {
        lookup[command.title] = command.id;
        return lookup;
      }, {});
    },
    async runCommand(id) {
      if (!this.client) {
        return;
      }
      const { taskId } = await this.client.post(`commands/run/${id}`);
      if (!taskId) {
        return;
      }
      this.addActiveCommand(id, taskId);
    },
    async getMacroId(macroName) {
      var _a;
      if (!this.macros) {
        await this.loadMacros();
      }
      return (_a = this.macros) == null ? void 0 : _a[macroName];
    },
    async getCommandId(commandName) {
      var _a;
      if (!this.commands) {
        await this.loadCommands();
      }
      return (_a = this.commands) == null ? void 0 : _a[commandName];
    }
  },
  getters: {
    isRelativeMove: (state) => state.modal.distance === "G91",
    distanceUnit: (state) => state.modal.units === "G21" ? "mm" : "in",
    distances: (state) => jogDistances[state.distanceUnit],
    speeds: (state) => jogSpeeds[state.distanceUnit],
    speedFallback: (state) => state.distanceUnit === "mm" ? 500 : 75,
    hold: (state) => state.runState === cncStates.HOLD,
    paused: (state) => state.workflowState === workflowStates.PAUSE || state.feederState === feederStates.PAUSE,
    feedPaused: (state) => state.feederState === feederStates.PAUSE,
    idle: (state) => state.workflowState === workflowStates.IDLE && state.feederState === feederStates.IDLE,
    running: (state) => state.workflowState === workflowStates.RUNNING && state.feederState === feederStates.IDLE,
    alarm: (state) => state.runState === cncStates.ALARM,
    elapsedTimeText: (state) => {
      if (!state.elapsedTime) {
        return "";
      }
      return formatDuration(state.elapsedTime);
    },
    displayWpos: (state) => formatAxes(state.wpos, state.axes),
    displayMpos: (state) => formatAxes(state.mpos, state.axes),
    remainingTimeText: (state) => {
      if (!state.remainingTime) {
        return "";
      }
      return formatDuration(state.remainingTime);
    },
    alarmText: (state) => `${["Alarm", state.alarmReason, state.locked ? "Locked" : null].filter(Boolean).join("\n")}`,
    pauseText: (state) => pauseText(state.pauseReason, state.pauseMessage),
    feedPauseText: (state) => pauseText(state.feedPauseReason, state.feedPauseMessage),
    accelerations: (state) => {
      if (!state.settings) {
        return;
      }
      return axisOrder.map((axis, i) => {
        return state.settings[`$${i + 121}`];
      }).filter((x) => x != null);
    },
    axisLimits: (state) => {
      if (!state.settings) {
        return {};
      }
      return axisOrder.reduce((limits, axis, i) => {
        limits[axis] = state.settings[`$${i + 130}`];
        return limits;
      }, {});
    },
    ready: (state) => state.connected && state.feederState === feederStates.IDLE && (state.runState === cncStates.IDLE || state.runState === cncStates.JOG)
  }
});
const useFileListStore = pinia.defineStore({
  id: "file-list",
  state: () => ({
    rowOffset: 0,
    path: null,
    files: [],
    loaded: false,
    client: null
  }),
  actions: {
    setClient(client) {
      this.client = client;
    },
    scrollUp() {
      this.rowOffset = Math.max(0, this.rowOffset - 1);
    },
    scrollDown() {
      this.rowOffset += 1;
    },
    previousFolder() {
      this.files = [];
      this.path = this.path.split("/").slice(0, -1).join("/");
      this.loadFiles(this.path);
    },
    loadFolder(folder = null) {
      this.files = [];
      this.path = [this.path, folder].filter(Boolean).join("/");
      this.loadFiles(this.path);
    },
    async loadFiles(path2 = "") {
      if (!this.client) {
        return;
      }
      this.files = [];
      this.rowOffset = 0;
      const fileList = await this.client.fetch("watch/files", { path: path2 });
      if (fileList) {
        this.loaded = true;
        this.path = fileList.path;
        this.files = fileList.files;
      }
    }
  }
});
const lazyStore$1 = {
  get cnc() {
    delete this.cnc;
    return this.cnc = useCncStore();
  }
};
const useGcodeStore = pinia.defineStore({
  id: "gcode",
  state: () => ({
    name: null,
    gcode: null,
    geometry: null
  }),
  getters: {
    dimensions() {
      if (!this.geometry) {
        return;
      }
      const unit = lazyStore$1.cnc.distanceUnit;
      if (unit === "in") {
        return this.geometry.size;
      }
      const res = {};
      ["min", "max"].forEach((dimKey) => {
        res[dimKey] = Object.entries(this.geometry.size[dimKey]).reduce((res2, [k, v]) => {
          res2[k] = v * 25.4;
          return res2;
        }, {});
      });
      return Object.freeze(res);
    },
    displayRange() {
      if (!this.dimensions) {
        return;
      }
      const res = {};
      ["min", "max"].forEach((dimKey) => {
        res[dimKey] = Object.entries(this.dimensions[dimKey]).reduce((res2, [k, v]) => {
          res2[k] = v.toFixed(3);
          return res2;
        }, {});
      });
      return Object.freeze(res);
    },
    displayDimensions() {
      if (!this.dimensions) {
        return;
      }
      const dimensions = {
        width: (this.dimensions.max.x - this.dimensions.min.x).toFixed(3),
        depth: (this.dimensions.max.y - this.dimensions.min.y).toFixed(3),
        height: (this.dimensions.max.z - this.dimensions.min.z).toFixed(3)
      };
      return Object.freeze(dimensions);
    }
  },
  actions: {
    unload() {
      this.$reset();
    },
    setLoaded(filename, code) {
      this.name = filename;
      if (code !== this.gcode) {
        this.geometry = null;
      }
      this.gcode = code;
    },
    setGeometry(geometry) {
      this.geometry = Object.freeze(geometry);
    }
  }
});
const mergeConfigs = /* @__PURE__ */ __name((...configs) => {
  const config = Object.assign(...configs);
  addKey(config);
  return config;
}, "mergeConfigs");
const addKey = /* @__PURE__ */ __name((config) => {
  config["key"] = SparkMD5__default.default.hash(JSON.stringify(config));
  return config;
}, "addKey");
const useFileList = /* @__PURE__ */ __name(() => {
  const ui = useUiStore();
  const cnc = useCncStore();
  const gcode = useGcodeStore();
  const { buttons: buttonConfig } = useButtonStore();
  const fileList = useFileListStore();
  const rows = ui.rows;
  const columns = ui.columns;
  const fileButton = /* @__PURE__ */ __name((filename, file) => {
    var _a;
    const fullPath = [fileList.path, filename].filter(Boolean).join("/");
    const selected = fullPath === gcode.name;
    const configButton = (_a = buttonConfig.fileListFile) != null ? _a : {};
    const activeActions = cnc.idle ? [
      {
        action: "loadFile",
        arguments: [fullPath],
        event: "up"
      },
      {
        action: "backScene",
        event: "up"
      }
    ] : [];
    const defaultButton = {
      text: filename,
      textAlignment: "bottom left",
      bgColor: selected || !cnc.idle ? 8 : 5,
      icon: "default/small_document.png",
      actions: [
        {
          action: "fileDetails",
          arguments: [fullPath, file],
          event: "hold"
        },
        ...activeActions
      ]
    };
    return mergeConfigs(defaultButton, configButton, {
      actions: defaultButton.actions
    });
  }, "fileButton");
  const folderButton = /* @__PURE__ */ __name((path2) => {
    var _a;
    const configButton = (_a = buttonConfig.fileListFolder) != null ? _a : {};
    const defaultButton = {
      text: path2,
      icon: "default/small_folder.png",
      textAlignment: "bottom left",
      bgColor: 4,
      actions: [
        {
          action: "loadFolder",
          arguments: [path2],
          event: "up"
        }
      ]
    };
    return mergeConfigs(defaultButton, configButton, {
      actions: defaultButton.actions
    });
  }, "folderButton");
  const previousFolder = /* @__PURE__ */ __name(() => {
    var _a;
    const configButton = (_a = buttonConfig.fileListPreviousFolder) != null ? _a : {};
    const defaultButton = {
      text: "..",
      icon: "default/small_folder_open_back.png",
      textAlignment: "bottom center",
      bgColor: 4,
      disabled: (!fileList.path).toString(),
      actions: [
        {
          action: "previousFolder",
          event: "up"
        }
      ]
    };
    return mergeConfigs(defaultButton, configButton, {
      actions: defaultButton.actions
    });
  }, "previousFolder");
  const upArrow = /* @__PURE__ */ __name(() => {
    var _a;
    const configButton = (_a = buttonConfig.fileListUpArrow) != null ? _a : {};
    const defaultButton = {
      icon: "fluent-ui/caret_up.png",
      bgColor: 7,
      actions: [{ action: "fileListScrollUp" }],
      disabled: (fileList.rowOffset === 0).toString()
    };
    return mergeConfigs(defaultButton, configButton);
  }, "upArrow");
  const downArrow = /* @__PURE__ */ __name(() => {
    var _a;
    const configButton = (_a = buttonConfig.fileListDownArrow) != null ? _a : {};
    const firstRowColumnsReserved = 3;
    const columnsReserved = 1;
    const visibleColumns = columns - columnsReserved;
    const pages = 1 + Math.ceil((fileList.files.length - (columns - firstRowColumnsReserved)) / visibleColumns);
    const defaultButton = {
      icon: "fluent-ui/caret_down.png",
      bgColor: 7,
      actions: [{ action: "fileListScrollDown" }],
      disabled: (fileList.rowOffset + rows >= pages).toString()
    };
    return mergeConfigs(defaultButton, configButton);
  }, "downArrow");
  const sortMethod = vue.computed(() => {
    let comparitor;
    const defaultSort = /* @__PURE__ */ __name((a, b) => a.name.localeCompare(b.name), "defaultSort");
    switch (ui.fileDetailsSort) {
      case "alpha_asc":
        comparitor = defaultSort;
        break;
      case "alpha_desc":
        comparitor = /* @__PURE__ */ __name((a, b) => b.name.localeCompare(a.name), "comparitor");
        break;
      case "modified_desc":
        comparitor = /* @__PURE__ */ __name((a, b) => Date.parse(b.mtime) - Date.parse(a.mtime), "comparitor");
        break;
      case "created_desc":
        comparitor = /* @__PURE__ */ __name((a, b) => Date.parse(b.ctime) - Date.parse(a.ctime), "comparitor");
        break;
      default:
        comparitor = defaultSort;
    }
    return (a, b) => (a.type === "d" ? -1 : 1) - (b.type === "d" ? -1 : 1) || comparitor(a, b);
  });
  const allButtons = vue.computed(() => {
    var _a;
    const sortedFiles = [...fileList.files];
    sortedFiles.sort(sortMethod.value);
    const files = sortedFiles.map((file) => {
      if (file.type === "d") {
        return folderButton(file.name);
      } else {
        return fileButton(file.name, file);
      }
    });
    const list = [[null]];
    list[0].push(previousFolder());
    let reserved = 3;
    let row = 0;
    while (files.length) {
      (_a = list[row]) != null ? _a : list[row] = [];
      const existing = list[row].length;
      list[row].splice(existing, 0, ...files.splice(0, columns - reserved));
      row++;
      reserved = 1;
    }
    return list;
  });
  const buttons = vue.computed(() => {
    if (!allButtons) {
      return [];
    }
    const pagedList = allButtons.value.slice(fileList.rowOffset, fileList.rowOffset + rows) || [];
    pagedList[0] = [...pagedList[0]];
    pagedList[0][0] = buttonConfig.back;
    pagedList[0][columns - 1] = upArrow();
    if (rows > 2) {
      pagedList[1] = pagedList.length > 1 ? [...pagedList[1]] : [];
      pagedList[1][columns - 1] = buttonConfig.sortScene;
    }
    pagedList[rows - 1] = pagedList.length === rows ? [...pagedList[rows - 1]] : [];
    pagedList[rows - 1][columns - 1] = downArrow();
    return pagedList;
  });
  const loadFiles = /* @__PURE__ */ __name(() => {
    if (!fileList.loaded) {
      fileList.loadFiles();
    }
  }, "loadFiles");
  return {
    buttons,
    loadFiles
  };
}, "useFileList");
const bus = /* @__PURE__ */ __name(() => {
  const inst = mitt__default.default();
  return {
    on: inst.on,
    off: inst.off,
    emit: inst.emit,
    all: inst.all,
    once: (type, fn) => {
      const cb = /* @__PURE__ */ __name(() => {
        inst.off(type, cb);
        fn();
      }, "cb");
      return inst.on(type, cb);
    }
  };
}, "bus");
const useRunGcode = /* @__PURE__ */ __name((socket, port) => {
  const cnc = useCncStore();
  const command = /* @__PURE__ */ __name((...args) => {
    socket.emit("command", port, ...args);
  }, "command");
  const runGcode = /* @__PURE__ */ __name((...args) => {
    command("gcode", ...args);
  }, "runGcode");
  const withRelative = /* @__PURE__ */ __name((callback) => {
    const wasRelative = cnc.isRelativeMove;
    runGcode("G91");
    callback();
    if (!wasRelative) {
      runGcode("G90");
    }
  }, "withRelative");
  const withAbsolute = /* @__PURE__ */ __name((callback) => {
    const wasRelative = cnc.isRelativeMove;
    runGcode("G90");
    callback();
    if (wasRelative) {
      runGcode("G91");
    }
  }, "withAbsolute");
  return {
    runGcode,
    withAbsolute,
    withRelative
  };
}, "useRunGcode");
const CncActions = /* @__PURE__ */ __name((socket, port, machineConfig, actionBus, ackBus) => {
  const cnc = useCncStore();
  const ui = useUiStore();
  const axisSpeeds = (machineConfig == null ? void 0 : machineConfig.axisSpeeds) || {};
  const { runGcode, withRelative, withAbsolute } = useRunGcode(socket, port);
  const command = /* @__PURE__ */ __name((...args) => {
    socket.emit("command", port, ...args);
  }, "command");
  const jog = /* @__PURE__ */ __name((direction, axis) => {
    const distance = cnc.jogDistance;
    const signedDistance = direction === "-" ? -distance : distance;
    withRelative(() => {
      runGcode(`G0 ${axis}${signedDistance}`);
    });
  }, "jog");
  const smoothJogFrequency = ui.web ? 150 : 50;
  const jogState = {
    timer: null,
    speed: null,
    axes: {},
    jogging: false
  };
  const promiseTimer = /* @__PURE__ */ __name((delay) => {
    return new Promise((resolve) => {
      setTimeout(resolve, delay);
    });
  }, "promiseTimer");
  const getAxisComponents = /* @__PURE__ */ __name((speeds, magnitude) => {
    const active = speeds.map(([key, value]) => {
      return value !== 0 && [key, value];
    }).filter(Boolean);
    const diagonalComponent = Math.sqrt(active.map(([, value]) => value ** 2).reduce((sum, val) => sum + val, 0));
    return active.map(([key, direction]) => {
      return [key, direction * magnitude / diagonalComponent];
    });
  }, "getAxisComponents");
  const promiseAck = /* @__PURE__ */ __name(() => {
    return new Promise((resolve) => {
      ackBus.once("ok", resolve);
    });
  }, "promiseAck");
  const smoothJogReady = /* @__PURE__ */ __name((delay) => {
    return Promise.all([promiseTimer(delay), promiseAck()]);
  }, "smoothJogReady");
  const smoothJogIteration = /* @__PURE__ */ __name(async ({ reducedDelay = false }) => {
    if (!jogState.jogging) {
      return;
    }
    clearTimeout(jogState.cancelTimer);
    const activeAxes = Object.entries(jogState.axes);
    if (activeAxes.length === 0) {
      clearSmoothJog();
      return;
    }
    const throttledSpeed = jogState.speed * Math.max(...activeAxes.map(([, val]) => Math.abs(val)));
    const jogIncrementDistance = throttledSpeed * (smoothJogFrequency / 6e4);
    const axisDistances = getAxisComponents(activeAxes, jogIncrementDistance);
    const jogAxes = axisDistances.map(([axis, distance]) => `${axis}${distance}`).join(" ");
    if (!jogAxes) {
      clearSmoothJog();
      return;
    }
    const jogGcode = `$J=G91 ${jogAxes} F${throttledSpeed}`;
    jogState.ack = false;
    runGcode(jogGcode);
    jogState.cancelTimer = setTimeout(smoothJogTimeout, 1e3);
    const delay = smoothJogFrequency - (reducedDelay ? 50 : 0);
    smoothJogReady(delay).then(smoothJogIteration);
  }, "smoothJogIteration");
  const smoothJogTimeout = /* @__PURE__ */ __name(() => {
    clearSmoothJog();
  }, "smoothJogTimeout");
  const smoothJog = /* @__PURE__ */ __name((direction, axis) => {
    var _a;
    const directionModifier = direction === "-" ? -1 : 1;
    const speedModifier = (_a = axisSpeeds == null ? void 0 : axisSpeeds[axis]) != null ? _a : 1;
    const v = directionModifier * speedModifier;
    if (jogState.axes[axis] != null) {
      if (v !== jogState.axes[axis]) {
        delete jogState.axes[axis];
      }
      return;
    }
    jogState.axes[axis] = v;
    jogState.speed = cnc.jogSpeed;
    if (!jogState.jogging) {
      jogState.jogging = true;
      smoothJogIteration({ reducedDelay: true });
    }
  }, "smoothJog");
  const clearSmoothJog = /* @__PURE__ */ __name(() => {
    jogState.jogging = false;
    jogState.speed = 0;
  }, "clearSmoothJog");
  const stopSmoothJog = /* @__PURE__ */ __name((axis) => {
    delete jogState.axes[axis];
    if (Object.keys(jogState.axes).length === 0) {
      clearSmoothJog();
    }
  }, "stopSmoothJog");
  actionBus.on("jog", ({ direction, axis }) => {
    jog(direction, axis);
  });
  actionBus.on("smoothJog", ({ direction, axis }) => {
    smoothJog(direction, axis);
  });
  actionBus.on("stopSmoothJog", ({ axis }) => {
    stopSmoothJog(axis);
  });
  actionBus.on("gcode", (code) => {
    runGcode(code);
  });
  actionBus.on("absoluteWorkPosition", (code) => {
    withAbsolute(() => runGcode(`G0 ${code}`));
  });
  actionBus.on("command", (evt) => {
    command(evt.command, ...evt.args || []);
  });
  actionBus.on("machinePosition", (evt) => {
    if (!evt) {
      return;
    }
    runGcode(`G53 G0 ${evt}`);
  });
  return {};
}, "CncActions");
let fetchImplementation;
{
  fetchImplementation = require("node-fetch");
}
const CncApi = /* @__PURE__ */ __name((token, host, port, secure) => {
  const url = /* @__PURE__ */ __name((path2, params) => {
    const queryParams = new URLSearchParams({
      token,
      ...params
    });
    return `${secure ? "https" : "http"}://${host}:${port}/api/${path2}?${queryParams}`;
  }, "url");
  const apiFetch = /* @__PURE__ */ __name(async (path2, params) => {
    const fetchUrl = url(path2, params);
    const results = await fetchImplementation(fetchUrl);
    return results.json();
  }, "apiFetch");
  const apiPost = /* @__PURE__ */ __name(async (path2, params) => {
    const postUrl = url(path2, params);
    const results = await fetchImplementation(postUrl, { method: "POST" });
    return results.json();
  }, "apiPost");
  return {
    fetch: apiFetch,
    post: apiPost
  };
}, "CncApi");
class ConnectionError extends Error {
  constructor(message) {
    super(message);
    this.name = "ConnectionError";
  }
}
__name(ConnectionError, "ConnectionError");
const Connection = /* @__PURE__ */ __name(function(opts, token) {
  var _a, _b, _c, _d, _e, _f;
  const options2 = { ...opts };
  (_a = options2.baudrate) != null ? _a : options2.baudrate = 115200;
  (_b = options2.socketAddress) != null ? _b : options2.socketAddress = "localhost";
  (_c = options2.secure) != null ? _c : options2.secure = false;
  (_d = options2.socketPort) != null ? _d : options2.socketPort = 8e3;
  (_e = options2.controllerType) != null ? _e : options2.controllerType = "Grbl";
  (_f = options2.accessTokenLifetime) != null ? _f : options2.accessTokenLifetime = "30d";
  this.options = options2;
  this.emitter = mitt__default.default();
  this.token = token;
}, "Connection");
const proto = Connection.prototype;
proto.validate = function() {
  const { socketAddress, port, baudrate, controllerType } = this.options;
  const required = {
    token: this.token,
    socketAddress,
    port,
    baudrate,
    controllerType
  };
  Object.entries(required).forEach(([key, val]) => {
    if (!val) {
      throw new ConnectionError(`${key} is required and was empty`);
    }
  });
};
proto.on = function(...args) {
  this.emitter.on(...args);
};
proto.off = function(...args) {
  this.emitter.off(...args);
};
proto.debug = function() {
  this.debugEnabled = true;
  if (!this.socket || this.debugging) {
    return;
  }
  this.debugging = true;
  const socket = this.socket;
  socket.on("connecting", (a) => {
    console.debug("Connecting");
  });
  socket.on("connect", (data) => {
    console.log("connected", data);
  });
  socket.on("connect_error", (err) => {
    console.debug(`connect_error due to ${err.message}`, err);
  });
  socket.on("serialport:close", (data) => {
    console.debug("Closed serialport", data);
  });
  socket.on("controller:state", (data) => {
    console.debug("controller:state", data);
  });
  socket.on("close", () => {
    console.debug("Connection closed.");
  });
  socket.on("serialport:read", function(data) {
    console.debug("serialport:read", (data || "").trim());
  });
  socket.on("feeder:status", function(data) {
    console.debug("feeder:status", data);
  });
  socket.on("sender:status", function(data) {
    console.debug("sender:status", data);
  });
  socket.on("Grbl:state", function(data) {
    console.debug("Grbl:state", data);
  });
  socket.on("Grbl:settings", function(data) {
    console.debug("grbl:settings", data);
  });
  socket.on("gcode:load", function(name, gcode, context) {
    console.debug("gcode:load", name, gcode.length, context);
  });
  socket.on("gcode:unload", function() {
    console.debug("gcode:unload");
  });
  socket.on("workflow:state", function(data) {
    console.debug("workflow:state", data);
  });
};
proto.openPort = function(port, baudrate, controllerType) {
  this.socket.emit("open", port, {
    baudrate: Number(baudrate),
    controllerType
  });
};
proto.openSerialPort = function() {
  const { baudrate, controllerType, port } = this.options;
  this.openPort(port, baudrate, controllerType);
};
proto.connect = function() {
  const { socketAddress, socketPort, secure } = this.options;
  this.validate();
  return new Promise((resolve, reject) => {
    const url = `${secure ? "wss" : "ws"}://${socketAddress}:${socketPort}/`;
    const socket = io__default.default.connect(url, {
      query: `token=${this.token}`,
      transports: ["websocket"]
    });
    this.socket = socket;
    socket.on("connect_error", (e) => {
      var _a;
      console.error(e.type, e);
      reject(new Error((_a = e.description) == null ? void 0 : _a.message));
    });
    socket.on("connect", () => {
      resolve({ socket });
      this.openSerialPort();
    });
    socket.on("serialport:open", (options2 = {}) => {
      this.options = { ...this.options, options: options2 };
      console.log(`Connected to port ${options2.port}`);
    });
    socket.on("serialport:change", ({ port, inuse }) => {
      if (inuse) {
        this.openSerialPort();
        this.emitter.emit("reconnected", { port });
      } else {
        this.emitter.emit("disconnected");
      }
    });
    socket.on("error", (err) => {
      console.error("Connection error.", err);
      if (socket) {
        socket.destroy();
        this.socket = null;
        reject(err);
      }
    });
    socket.on("serialport:error", function(options2) {
      reject(new Error(`Error opening serial port "${options2.port}"`));
    });
  });
};
const Connection$1 = Connection;
program__default.default.version("0.9.0").option("-e, --example", "save an example config file and exit").option("-d, --directory <directory>", "path to configuration directory", process.cwd());
const cliOptions = /* @__PURE__ */ __name(() => {
  program__default.default.parse();
  const options2 = program__default.default.opts();
  if (options2.example) {
    fs__default.default.copyFileSync(path__default.default.join(__dirname, "config.example.json"), "config.example.json");
    console.log(`${process.cwd()}/config.example.json saved`);
    process.exit();
  }
  return options2;
}, "cliOptions");
const fsModule = require("fs");
const path = require("path");
const { Worker } = require("worker_threads");
const fs = fsModule.promises;
const chalk = require("chalk");
const { performance: nodePerformance } = require("perf_hooks");
const getUserHome = /* @__PURE__ */ __name(function() {
  return process.env[process.platform === "win32" ? "USERPROFILE" : "HOME"];
}, "getUserHome");
const getSecret = /* @__PURE__ */ __name(async (secret) => {
  if (secret) {
    return secret;
  }
  try {
    const cncrc = path.resolve(getUserHome(), ".cncrc");
    const data = await fs.readFile(cncrc, "utf8");
    const config = JSON.parse(data);
    return config.secret;
  } catch (e) {
  }
}, "getSecret");
const fetchConfig = /* @__PURE__ */ __name(async () => {
  const options2 = getOptions();
  const configPath = path.resolve(options2.directory, "config.json");
  const data = await fs.readFile(configPath, "utf8");
  return JSON.parse(data);
}, "fetchConfig");
let options;
const getOptions = /* @__PURE__ */ __name(() => {
  if (options) {
    return options;
  }
  options = cliOptions();
  return options;
}, "getOptions");
const getAccessToken = /* @__PURE__ */ __name(async (secret, expiration) => {
  secret = await getSecret(secret);
  if (!secret) {
    console.error(chalk.red("Secret is required, see --secret command line option"));
    process.exit(1);
  }
  const payload = { id: "", name: "cncjs-pendant" };
  return jwt__default.default.sign(payload, secret, {
    expiresIn: expiration
  });
}, "getAccessToken");
const GcodeWorker = /* @__PURE__ */ __name(() => {
  return new Worker(path.resolve(__dirname, "gcode-thread-worker.js"), {
    type: "module"
  });
}, "GcodeWorker");
const OutlineWorker = /* @__PURE__ */ __name(() => {
  return new Worker(path.resolve(__dirname, "outline-thread-worker.js"), {
    type: "module"
  });
}, "OutlineWorker");
const onWorkerEvent = /* @__PURE__ */ __name((worker, event, callback) => {
  worker.on(event, callback);
}, "onWorkerEvent");
const offWorkerEvent = /* @__PURE__ */ __name((worker, event, callback) => {
  worker.removeListener(event, callback);
}, "offWorkerEvent");
const performance = nodePerformance;
const gcodeWorker = GcodeWorker();
const StateFeeder = /* @__PURE__ */ __name((socket, ackBus) => {
  const cnc = useCncStore();
  const ui = useUiStore();
  const gcode = useGcodeStore();
  const listeners = {
    "Grbl:state": (data) => {
      const { wpos, mpos, ov, spindle, feedrate } = data.status;
      const { modal } = data.parserstate;
      cnc.setOverrides(...ov);
      cnc.setWpos(wpos);
      cnc.setMpos(mpos);
      cnc.setModal(modal);
      cnc.setFeedrate(feedrate);
      cnc.setSpindleRpm(spindle);
      cnc.setRunState(data.status.activeState);
    },
    "serialport:read": (data) => {
      switch (true) {
        case data === "ok":
        case data.startsWith("error:15"):
          ackBus.emit("ok");
          break;
        case data.startsWith("ALARM:"):
          cnc.setAlarm(data.substring(6));
          break;
        case data.includes("'$X' to unlock"):
          cnc.setLocked();
          break;
        case data.includes("Unlocked"):
          cnc.setLocked(false);
      }
    },
    "serialport:change": ({ inuse }) => {
      cnc.setConnected(inuse);
    },
    "serialport:open": () => {
      cnc.setConnected(true);
    },
    "Grbl:settings": ({ parameters, settings, version }) => {
      cnc.setSettings(settings);
      cnc.setVersion(version);
    },
    "gcode:load": (file, code) => {
      gcodeWorker.postMessage({ name: file, gcode: code, limit: ui.gcodeLimit });
      gcode.setLoaded(file, code);
    },
    "gcode:unload": () => {
      gcode.unload();
    },
    "sender:status": (status) => {
      cnc.setElapsedTime(status.elapsedTime * 1e-3);
      cnc.setRemainingTime(status.remainingTime * 1e-3);
      if (!status.hold) {
        return;
      }
      let data, msg, err;
      if (status.holdReason) {
        ({ data, msg, err } = status.holdReason);
      }
      cnc.setPause(data, msg);
      if (err) {
        cnc.setError(msg);
      }
    },
    "task:finish": (task, code) => {
      cnc.clearActiveCommand(task);
    },
    "feeder:status": (status) => {
      if (!status.hold) {
        cnc.clearFeedHold();
        return;
      }
      let data, msg, err;
      if (status.holdReason) {
        ({ data, msg, err } = status.holdReason);
      }
      cnc.setFeedHold(data, msg);
      if (err) {
        console.error(err);
      }
    },
    "workflow:state": (status) => {
      cnc.setWorkflowState(status);
    },
    connect_error: () => {
      if (cnc.connected) {
        cnc.setConnected(false);
      }
    },
    connect: () => {
      cnc.clearActiveCommands();
    }
  };
  const workerListeners = {
    message: (e) => {
      const { name, geometry } = e.data;
      if (gcode.name === name) {
        gcode.setGeometry(geometry);
      }
    }
  };
  Object.entries(listeners).forEach(([event, listener]) => {
    socket.on(event, listener);
  });
  Object.entries(workerListeners).forEach(([event, listener]) => {
    onWorkerEvent(gcodeWorker, event, listener);
  });
  const destroy = /* @__PURE__ */ __name(() => {
    Object.entries(listeners).forEach(([event, listener]) => {
      socket.off(event, listener);
    });
    Object.entries(workerListeners).forEach(([event, listener]) => {
      offWorkerEvent(gcodeWorker, event, listener);
    });
  }, "destroy");
  return {
    destroy
  };
}, "StateFeeder");
const outlineWorker = OutlineWorker();
const lazyStore = /* @__PURE__ */ __name(() => {
  return {
    get ui() {
      delete this.ui;
      return this.ui = useUiStore();
    },
    get cnc() {
      delete this.cnc;
      return this.cnc = useCncStore();
    },
    get fileList() {
      delete this.fileList;
      return this.fileList = useFileListStore();
    },
    get gcode() {
      delete this.gcode;
      return this.gcode = useGcodeStore();
    }
  };
}, "lazyStore");
const machineCommands = /* @__PURE__ */ new Set([
  "absoluteWorkPosition",
  "absoluteMachinePosition",
  "clearGcode",
  "enterPosition",
  "enterWcs",
  "gcode",
  "goto",
  "homing",
  "jog",
  "loadDetailFile",
  "loadGcode",
  "macro",
  "resetFeedrate",
  "resetRapids",
  "resetSpindle",
  "decreaseFeedrate",
  "increaseFeedrate",
  "setRapids",
  "increaseSpindle",
  "decreaseSpindle",
  "startSmoothJog",
  "stopSmoothJog"
]);
const runningCommands = /* @__PURE__ */ new Set([
  "decreaseFeedrate",
  "decreaseSpindle",
  "increaseFeedrate",
  "increaseSpindle",
  "resetFeedrate",
  "resetRapids",
  "resetSpindle",
  "setRapids"
]);
const alarmCommands = /* @__PURE__ */ new Set(["homing", "unlock"]);
const ButtonActions = /* @__PURE__ */ __name((actionBus, connectionBus) => {
  const store = lazyStore();
  const backScene = /* @__PURE__ */ __name((count = 1) => {
    store.ui.goBack(count);
  }, "backScene");
  const swapScene = /* @__PURE__ */ __name((scene) => {
    store.ui.swapScene(scene);
  }, "swapScene");
  const navigate = /* @__PURE__ */ __name((scene) => {
    store.ui.goToScene(scene);
  }, "navigate");
  const enterWcs = /* @__PURE__ */ __name((axis, scene = "numpad") => {
    const label = `${store.cnc.modal.wcs}
${axis.toUpperCase()} offset`;
    store.ui.startInput(store.cnc.wpos[axis], label, scene, (result) => {
      gcode(`G10 L20 P1 ${axis}${result}`);
    });
  }, "enterWcs");
  const enterPosition = /* @__PURE__ */ __name((axis, scene = "numpad") => {
    const label = `${store.cnc.modal.wcs}
Go to ${axis.toUpperCase()}`;
    store.ui.startInput(store.cnc.wpos[axis], label, scene, (result) => {
      absoluteWorkPosition(`${axis}${result}`);
    });
  }, "enterPosition");
  const completeInput = /* @__PURE__ */ __name(() => {
    store.ui.completeInput();
  }, "completeInput");
  const input = /* @__PURE__ */ __name((chars) => {
    store.ui.addInput(chars);
  }, "input");
  const inputCommand = /* @__PURE__ */ __name((command2) => {
    const input2 = store.ui.input;
    if (command2 === "backspace") {
      input2.value = input2.value.slice(0, -1);
    } else if (command2 === "toggleSign") {
      input2.value = input2.value.startsWith("-") ? input2.value.slice(1) : "-" + input2.value;
    }
  }, "inputCommand");
  const jog = /* @__PURE__ */ __name((direction, axis) => {
    actionBus.emit("jog", { direction, axis });
  }, "jog");
  const jogDistance = /* @__PURE__ */ __name((sign) => {
    if (sign === "-") {
      store.cnc.decreaseJogDistance();
    } else {
      store.cnc.increaseJogDistance();
    }
  }, "jogDistance");
  const jogSpeed = /* @__PURE__ */ __name((sign) => {
    if (sign === "-") {
      store.cnc.decreaseJogSpeed();
    } else {
      store.cnc.increaseJogSpeed();
    }
  }, "jogSpeed");
  const run2 = /* @__PURE__ */ __name(() => {
    if (store.cnc.idle) {
      command("gcode:start");
      return;
    }
    if (store.cnc.paused) {
      command("gcode:resume");
    }
  }, "run");
  const pause = /* @__PURE__ */ __name(() => {
    if (store.cnc.running) {
      command("gcode:pause");
    }
  }, "pause");
  const stop = /* @__PURE__ */ __name(() => {
    if (store.cnc.paused) {
      command("gcode:stop", { force: true });
    }
  }, "stop");
  const clearGcode = /* @__PURE__ */ __name(() => {
    if (store.cnc.idle && store.gcode.gcode) {
      command("gcode:unload");
    }
  }, "clearGcode");
  const startSmoothJog = /* @__PURE__ */ __name((direction, axis) => {
    actionBus.emit("smoothJog", { direction, axis });
  }, "startSmoothJog");
  const stopSmoothJog = /* @__PURE__ */ __name((direction, axis) => {
    actionBus.emit("stopSmoothJog", { direction, axis });
  }, "stopSmoothJog");
  const gcode = /* @__PURE__ */ __name((code) => {
    actionBus.emit("gcode", code);
  }, "gcode");
  const absoluteMachinePosition = /* @__PURE__ */ __name((code) => {
    actionBus.emit("machinePosition", code);
  }, "absoluteMachinePosition");
  const absoluteWorkPosition = /* @__PURE__ */ __name((code) => {
    actionBus.emit("absoluteWorkPosition", code);
  }, "absoluteWorkPosition");
  const command = /* @__PURE__ */ __name((cmd, ...args) => {
    actionBus.emit("command", { command: cmd, args });
  }, "command");
  const reset = /* @__PURE__ */ __name(() => {
    command("reset");
  }, "reset");
  const connect = /* @__PURE__ */ __name(() => {
    if (!store.cnc.connected) {
      store.cnc.setConnecting();
      connectionBus.emit("connect");
    }
  }, "connect");
  const userCommand = /* @__PURE__ */ __name(async (commandId, commandName) => {
    if (!commandId && commandName) {
      commandId = await store.cnc.getCommandId(commandName);
      if (!commandId) {
        console.error(`Requested command "${commandName}" could not be found`);
        return;
      }
    }
    return store.cnc.runCommand(commandId);
  }, "userCommand");
  const macro = /* @__PURE__ */ __name(async (macroId, macroName) => {
    if (!macroId && macroName) {
      macroId = await store.cnc.getMacroId(macroName);
      if (!macroId) {
        console.error(`Requested macro "${macroName}" could not be found`);
        return;
      }
    }
    command("macro:run", macroId);
  }, "macro");
  const homing = /* @__PURE__ */ __name(() => {
    command("homing");
  }, "homing");
  const unlock = /* @__PURE__ */ __name(() => {
    command("unlock");
  }, "unlock");
  const hold = /* @__PURE__ */ __name(() => {
    command("feedhold");
  }, "hold");
  const unhold = /* @__PURE__ */ __name(() => {
    command("cyclestart");
  }, "unhold");
  const fullscreen = /* @__PURE__ */ __name(() => {
    if (document.fullscreenElement) {
      document.exitFullscreen();
    } else {
      document.documentElement.requestFullscreen();
    }
  }, "fullscreen");
  const fileDetails = /* @__PURE__ */ __name((path2, file) => {
    store.ui.fileDetailsPath = path2;
    store.ui.fileDetails = file;
    navigate("fileDetails");
  }, "fileDetails");
  const previousFolder = /* @__PURE__ */ __name(() => {
    store.fileList.previousFolder();
  }, "previousFolder");
  const loadFolder = /* @__PURE__ */ __name((path2) => {
    store.fileList.loadFolder(path2);
  }, "loadFolder");
  const refreshWatchFolder = /* @__PURE__ */ __name(async () => {
    await store.fileList.loadFiles();
    navigate("gcodeList");
  }, "refreshWatchFolder");
  const toggleFeedrateInterval = /* @__PURE__ */ __name(() => {
    store.ui.toggleFeedrateInterval();
  }, "toggleFeedrateInterval");
  const increaseFeedrate = /* @__PURE__ */ __name(() => {
    command("feedOverride", store.ui.feedrateInterval);
  }, "increaseFeedrate");
  const decreaseFeedrate = /* @__PURE__ */ __name(() => {
    command("feedOverride", -store.ui.feedrateInterval);
  }, "decreaseFeedrate");
  const resetFeedrate = /* @__PURE__ */ __name(() => {
    command("feedOverride", 0);
  }, "resetFeedrate");
  const toggleSpindleInterval = /* @__PURE__ */ __name(() => {
    store.ui.toggleSpindleInterval();
  }, "toggleSpindleInterval");
  const increaseSpindle = /* @__PURE__ */ __name(() => {
    command("spindleOverride", store.ui.spindleInterval);
  }, "increaseSpindle");
  const decreaseSpindle = /* @__PURE__ */ __name(() => {
    command("spindleOverride", -store.ui.spindleInterval);
  }, "decreaseSpindle");
  const resetSpindle = /* @__PURE__ */ __name(() => {
    command("spindleOverride", 0);
  }, "resetSpindle");
  const setRapids = /* @__PURE__ */ __name((speed) => {
    if (["25", "50", "100"].includes(speed)) {
      command("rapidOverride", Number(speed));
    }
  }, "setRapids");
  const brightness = /* @__PURE__ */ __name((sign) => {
    if (sign === "-") {
      store.ui.decreaseBrightness();
    } else {
      store.ui.increaseBrightness();
    }
  }, "brightness");
  const loadFile = /* @__PURE__ */ __name((path2) => {
    store.gcode.setLoaded(path2, null);
    command("watchdir:load", path2);
  }, "loadFile");
  const loadDetailFile = /* @__PURE__ */ __name(() => {
    if (!store.ui.fileDetailsPath) {
      return;
    }
    loadFile(store.ui.fileDetailsPath);
  }, "loadDetailFile");
  const fileListScrollUp = /* @__PURE__ */ __name(() => {
    store.fileList.scrollUp();
  }, "fileListScrollUp");
  const fileListScrollDown = /* @__PURE__ */ __name(() => {
    store.fileList.scrollDown();
  }, "fileListScrollDown");
  const sortDetails = /* @__PURE__ */ __name((sort) => {
    store.ui.fileDetailsSort = sort;
  }, "sortDetails");
  const goto = /* @__PURE__ */ __name((x, y, z, a, b, c) => {
    const limits = store.cnc.axisLimits;
    const move = Object.entries({ x, y, z, a, b, c }).map(([axis, position]) => {
      if (position == null ? void 0 : position.endsWith("%")) {
        if (limits[axis]) {
          const absPosition = parseFloat(position) * -0.01 * limits[axis];
          return `${axis}${absPosition.toFixed(5)}`;
        }
      } else if (position != null) {
        return `${axis}${position}`;
      }
    }).filter(Boolean).join(" ");
    actionBus.emit("machinePosition", move);
  }, "goto");
  const startFeed = /* @__PURE__ */ __name(() => {
    command("feeder:start");
  }, "startFeed");
  const stopFeed = /* @__PURE__ */ __name(() => {
    command("feeder:stop");
  }, "stopFeed");
  const setUserFlag = /* @__PURE__ */ __name((id, value) => {
    store.ui.setUserFlag(id, value);
  }, "setUserFlag");
  const clearUserFlag = /* @__PURE__ */ __name((id, value) => {
    store.ui.clearUserFlag(id, value);
  }, "clearUserFlag");
  const toggleUserFlag = /* @__PURE__ */ __name((id) => {
    store.ui.setUserFlag(id, !store.ui.userFlags[id]);
  }, "toggleUserFlag");
  const outline = /* @__PURE__ */ __name(() => {
    if (store.cnc.idle && store.gcode.gcode) {
      store.cnc.addActiveWorker("outline");
      outlineWorker.postMessage({
        gcode: store.gcode.gcode,
        x: parseFloat(store.cnc.wpos.x),
        y: parseFloat(store.cnc.wpos.y),
        name: store.gcode.name,
        feedbackUnits: store.cnc.settings["$13"] === "1" ? "in" : "mm"
      });
    }
  }, "outline");
  const actionTypes = {};
  const actions = {
    absoluteWorkPosition,
    absoluteMachinePosition,
    backScene,
    brightness,
    clearGcode,
    clearUserFlag,
    command: userCommand,
    completeInput,
    connect,
    decreaseFeedrate,
    decreaseSpindle,
    enterPosition,
    enterWcs,
    fileDetails,
    loadDetailFile,
    fileListScrollDown,
    fileListScrollUp,
    fullscreen,
    gcode,
    goto,
    hold,
    homing,
    increaseFeedrate,
    increaseSpindle,
    input,
    inputCommand,
    jog,
    jogDistance,
    jogSpeed,
    loadFile,
    loadFolder,
    macro,
    navigate,
    outline,
    pause,
    previousFolder,
    refreshWatchFolder,
    reset,
    resetFeedrate,
    resetSpindle,
    run: run2,
    setRapids,
    setUserFlag,
    sortDetails,
    startFeed,
    startSmoothJog,
    stop,
    stopFeed,
    stopSmoothJog,
    swapScene,
    toggleFeedrateInterval,
    toggleSpindleInterval,
    toggleUserFlag,
    unhold,
    unlock
  };
  const ensureHandler = /* @__PURE__ */ __name((cfg) => {
    if (!cfg) {
      return;
    }
    const smoothJog = cfg.filter((action) => {
      return (action == null ? void 0 : action.action) === "startSmoothJog";
    });
    if (smoothJog.length > 0) {
      return smoothJog.map((action) => {
        return {
          action: stopSmoothJog,
          arguments: action.arguments
        };
      });
    }
  }, "ensureHandler");
  const enabled = /* @__PURE__ */ __name((cfg) => store.cnc.ready || !(cfg == null ? void 0 : cfg.every((action) => machineCommands.has(action.action))) || store.cnc.running && (cfg == null ? void 0 : cfg.some((action) => runningCommands.has(action.action))) || store.cnc.alarm && (cfg == null ? void 0 : cfg.some((action) => alarmCommands.has(action.action))), "enabled");
  const getCallback = /* @__PURE__ */ __name((cfg) => {
    return cfg.action && actions[cfg.action];
  }, "getCallback");
  const getHandlers = /* @__PURE__ */ __name((cfg) => {
    if (!cfg) {
      return {};
    }
    return cfg.reduce((grouped, action) => {
      var _a, _b;
      const callback = getCallback(action);
      if (!callback) {
        return grouped;
      }
      const event = (_a = action.event) != null ? _a : "down";
      (_b = grouped[event]) != null ? _b : grouped[event] = [];
      grouped[event].push({
        action: callback,
        type: actionTypes[action.action],
        arguments: action.arguments
      });
      return grouped;
    }, {});
  }, "getHandlers");
  const workerListeners = {
    error: (e) => {
      store.cnc.clearActiveWorker("outline");
      console.error(e.data.message);
    },
    message: (e) => {
      store.cnc.clearActiveWorker("outline");
      const { perimeter, name } = e.data;
      if (store.gcode.name === name) {
        gcode(perimeter);
      }
    }
  };
  Object.entries(workerListeners).forEach(([event, listener]) => {
    onWorkerEvent(outlineWorker, event, listener);
  });
  const destroy = /* @__PURE__ */ __name(() => {
    Object.entries(workerListeners).forEach(([event, listener]) => {
      offWorkerEvent(outlineWorker, event, listener);
    });
  }, "destroy");
  return {
    getHandlers,
    ensureHandler,
    enabled,
    destroy
  };
}, "ButtonActions");
function getDefaultExportFromCjs(x) {
  return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, "default") ? x["default"] : x;
}
__name(getDefaultExportFromCjs, "getDefaultExportFromCjs");
var serviceprovider = { exports: {} };
var basicprovider = { exports: {} };
(function(module2, exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  __name(_classCallCheck, "_classCallCheck");
  var BasicProvider = /* @__PURE__ */ __name(function BasicProvider2(container2, id, resource) {
    var options2 = arguments.length <= 3 || arguments[3] === void 0 ? {} : arguments[3];
    _classCallCheck(this, BasicProvider2);
    if (!container2 || container2.constructor.name !== "Cation") {
      throw new Error("Invalid container instance");
    }
    if (typeof options2.args === "undefined") {
      options2.args = [];
    }
    if (typeof options2.decorators === "undefined") {
      options2.decorators = [];
    }
    if (typeof options2.tags === "undefined") {
      options2.tags = [];
    }
    this.container = container2;
    this.id = id;
    this.resource = resource;
    this.options = options2;
  }, "BasicProvider");
  exports["default"] = BasicProvider;
  module2.exports = exports["default"];
})(basicprovider, basicprovider.exports);
var service = {};
/*! Service helper */
Object.defineProperty(service, "__esModule", {
  value: true
});
var resolveDependencies_1 = service.resolveDependencies = resolveDependencies;
function resolveDependencies(container2) {
  var resourceArguments = arguments.length <= 1 || arguments[1] === void 0 ? [] : arguments[1];
  return new Promise(function(resolve, reject) {
    if (!resourceArguments.length) {
      return resolve([]);
    }
    var resolvedArguments = resourceArguments.map(function(stringArgument) {
      return resolveArgument(container2, stringArgument);
    });
    Promise.all(resolvedArguments).then(function(serviceArgs) {
      return resolve(serviceArgs);
    })["catch"](function(error) {
      return reject(error);
    });
  });
}
__name(resolveDependencies, "resolveDependencies");
function resolveArgument(container2, argument) {
  var actions = {
    "@": /* @__PURE__ */ __name(function _(value) {
      return container2.get(value);
    }, "_"),
    "default": /* @__PURE__ */ __name(function _default(value) {
      return value;
    }, "_default")
  };
  if (typeof argument !== "string") {
    return argument;
  }
  var serviceIdRegex = /^([@|\\\\/(?=@)]?)(.+)$/;
  var matches = serviceIdRegex.exec(argument);
  var resolverAction = matches[1];
  var resolverValue = matches[2];
  if (typeof actions[resolverAction] === "undefined") {
    return actions["default"](resolverValue);
  }
  return actions[resolverAction](resolverValue);
}
__name(resolveArgument, "resolveArgument");
(function(module2, exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  var _createClass = function() {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor)
          descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }
    __name(defineProperties, "defineProperties");
    return function(Constructor, protoProps, staticProps) {
      if (protoProps)
        defineProperties(Constructor.prototype, protoProps);
      if (staticProps)
        defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();
  var _get = /* @__PURE__ */ __name(function get(_x2, _x3, _x4) {
    var _again = true;
    _function:
      while (_again) {
        var object = _x2, property = _x3, receiver = _x4;
        _again = false;
        if (object === null)
          object = Function.prototype;
        var desc = Object.getOwnPropertyDescriptor(object, property);
        if (desc === void 0) {
          var parent = Object.getPrototypeOf(object);
          if (parent === null) {
            return void 0;
          } else {
            _x2 = parent;
            _x3 = property;
            _x4 = receiver;
            _again = true;
            desc = parent = void 0;
            continue _function;
          }
        } else if ("value" in desc) {
          return desc.value;
        } else {
          var getter = desc.get;
          if (getter === void 0) {
            return void 0;
          }
          return getter.call(receiver);
        }
      }
  }, "get");
  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};
      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key))
            newObj[key] = obj[key];
        }
      }
      newObj["default"] = obj;
      return newObj;
    }
  }
  __name(_interopRequireWildcard, "_interopRequireWildcard");
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : { "default": obj };
  }
  __name(_interopRequireDefault, "_interopRequireDefault");
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  __name(_classCallCheck, "_classCallCheck");
  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } });
    if (superClass)
      Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }
  __name(_inherits, "_inherits");
  var _basicprovider = basicprovider.exports;
  var _basicprovider2 = _interopRequireDefault(_basicprovider);
  var _helpersService = service;
  var util = _interopRequireWildcard(_helpersService);
  var ServiceProvider2 = function(_BasicProvider) {
    _inherits(ServiceProvider3, _BasicProvider);
    function ServiceProvider3(container2, id, resource) {
      var options2 = arguments.length <= 3 || arguments[3] === void 0 ? {} : arguments[3];
      _classCallCheck(this, ServiceProvider3);
      _get(Object.getPrototypeOf(ServiceProvider3.prototype), "constructor", this).call(this, container2, id, resource, options2);
    }
    __name(ServiceProvider3, "ServiceProvider");
    _createClass(ServiceProvider3, [{
      key: "get",
      value: /* @__PURE__ */ __name(function get() {
        var _this = this;
        var serviceDepsPromise = util.resolveDependencies(this.container, this.options.args);
        return serviceDepsPromise.then(function(serviceDeps) {
          serviceDeps.unshift(_this.resource);
          var Resource = _this.resource.bind.apply(_this.resource, serviceDeps);
          return new Resource();
        });
      }, "get")
    }]);
    return ServiceProvider3;
  }(_basicprovider2["default"]);
  exports["default"] = ServiceProvider2;
  module2.exports = exports["default"];
})(serviceprovider, serviceprovider.exports);
const ServiceProvider = /* @__PURE__ */ getDefaultExportFromCjs(serviceprovider.exports);
class MethodProvider extends ServiceProvider {
  constructor(container2, id, resource, options2) {
    super(container2, id, resource, options2);
  }
  get() {
    let serviceDepsPromise = resolveDependencies_1(this.container, this.options.args);
    return serviceDepsPromise.then((serviceDeps) => {
      serviceDeps.unshift(this.resource);
      const Resource = this.resource.bind.apply(this.resource, serviceDeps);
      return Resource();
    });
  }
}
__name(MethodProvider, "MethodProvider");
const MethodProvider$1 = MethodProvider;
var cation = {};
var factoryprovider = { exports: {} };
(function(module2, exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  var _createClass = function() {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor)
          descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }
    __name(defineProperties, "defineProperties");
    return function(Constructor, protoProps, staticProps) {
      if (protoProps)
        defineProperties(Constructor.prototype, protoProps);
      if (staticProps)
        defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();
  var _get = /* @__PURE__ */ __name(function get(_x2, _x3, _x4) {
    var _again = true;
    _function:
      while (_again) {
        var object = _x2, property = _x3, receiver = _x4;
        _again = false;
        if (object === null)
          object = Function.prototype;
        var desc = Object.getOwnPropertyDescriptor(object, property);
        if (desc === void 0) {
          var parent = Object.getPrototypeOf(object);
          if (parent === null) {
            return void 0;
          } else {
            _x2 = parent;
            _x3 = property;
            _x4 = receiver;
            _again = true;
            desc = parent = void 0;
            continue _function;
          }
        } else if ("value" in desc) {
          return desc.value;
        } else {
          var getter = desc.get;
          if (getter === void 0) {
            return void 0;
          }
          return getter.call(receiver);
        }
      }
  }, "get");
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : { "default": obj };
  }
  __name(_interopRequireDefault, "_interopRequireDefault");
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  __name(_classCallCheck, "_classCallCheck");
  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } });
    if (superClass)
      Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }
  __name(_inherits, "_inherits");
  var _basicprovider = basicprovider.exports;
  var _basicprovider2 = _interopRequireDefault(_basicprovider);
  var FactoryProvider = function(_BasicProvider) {
    _inherits(FactoryProvider2, _BasicProvider);
    function FactoryProvider2(container2, id, resource) {
      var options2 = arguments.length <= 3 || arguments[3] === void 0 ? {} : arguments[3];
      _classCallCheck(this, FactoryProvider2);
      _get(Object.getPrototypeOf(FactoryProvider2.prototype), "constructor", this).call(this, container2, id, resource, options2);
    }
    __name(FactoryProvider2, "FactoryProvider");
    _createClass(FactoryProvider2, [{
      key: "get",
      value: /* @__PURE__ */ __name(function get() {
        return this.resource.apply(this.resource, [this.container]);
      }, "get")
    }]);
    return FactoryProvider2;
  }(_basicprovider2["default"]);
  exports["default"] = FactoryProvider;
  module2.exports = exports["default"];
})(factoryprovider, factoryprovider.exports);
var staticprovider = { exports: {} };
(function(module2, exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  var _createClass = function() {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor)
          descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }
    __name(defineProperties, "defineProperties");
    return function(Constructor, protoProps, staticProps) {
      if (protoProps)
        defineProperties(Constructor.prototype, protoProps);
      if (staticProps)
        defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();
  var _get = /* @__PURE__ */ __name(function get(_x2, _x3, _x4) {
    var _again = true;
    _function:
      while (_again) {
        var object = _x2, property = _x3, receiver = _x4;
        _again = false;
        if (object === null)
          object = Function.prototype;
        var desc = Object.getOwnPropertyDescriptor(object, property);
        if (desc === void 0) {
          var parent = Object.getPrototypeOf(object);
          if (parent === null) {
            return void 0;
          } else {
            _x2 = parent;
            _x3 = property;
            _x4 = receiver;
            _again = true;
            desc = parent = void 0;
            continue _function;
          }
        } else if ("value" in desc) {
          return desc.value;
        } else {
          var getter = desc.get;
          if (getter === void 0) {
            return void 0;
          }
          return getter.call(receiver);
        }
      }
  }, "get");
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : { "default": obj };
  }
  __name(_interopRequireDefault, "_interopRequireDefault");
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  __name(_classCallCheck, "_classCallCheck");
  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } });
    if (superClass)
      Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }
  __name(_inherits, "_inherits");
  var _basicprovider = basicprovider.exports;
  var _basicprovider2 = _interopRequireDefault(_basicprovider);
  var StaticProvider = function(_BasicProvider) {
    _inherits(StaticProvider2, _BasicProvider);
    function StaticProvider2(container2, id, resource) {
      var options2 = arguments.length <= 3 || arguments[3] === void 0 ? {} : arguments[3];
      _classCallCheck(this, StaticProvider2);
      _get(Object.getPrototypeOf(StaticProvider2.prototype), "constructor", this).call(this, container2, id, resource, options2);
    }
    __name(StaticProvider2, "StaticProvider");
    _createClass(StaticProvider2, [{
      key: "get",
      value: /* @__PURE__ */ __name(function get() {
        var _this = this;
        return new Promise(function(resolve) {
          return resolve(_this.resource);
        });
      }, "get")
    }]);
    return StaticProvider2;
  }(_basicprovider2["default"]);
  exports["default"] = StaticProvider;
  module2.exports = exports["default"];
})(staticprovider, staticprovider.exports);
var decorator = {};
/*! Decorator Helper */
Object.defineProperty(decorator, "__esModule", {
  value: true
});
decorator.reducer = reducer;
function reducer(instance, decoratorFunction) {
  return decoratorFunction(instance);
}
__name(reducer, "reducer");
/*! Module dependencies */
(function(exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  var _slicedToArray = function() {
    function sliceIterator(arr, i) {
      var _arr = [];
      var _n = true;
      var _d = false;
      var _e = void 0;
      try {
        for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
          _arr.push(_s.value);
          if (i && _arr.length === i)
            break;
        }
      } catch (err) {
        _d = true;
        _e = err;
      } finally {
        try {
          if (!_n && _i["return"])
            _i["return"]();
        } finally {
          if (_d)
            throw _e;
        }
      }
      return _arr;
    }
    __name(sliceIterator, "sliceIterator");
    return function(arr, i) {
      if (Array.isArray(arr)) {
        return arr;
      } else if (Symbol.iterator in Object(arr)) {
        return sliceIterator(arr, i);
      } else {
        throw new TypeError("Invalid attempt to destructure non-iterable instance");
      }
    };
  }();
  var _createClass = function() {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor)
          descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }
    __name(defineProperties, "defineProperties");
    return function(Constructor, protoProps, staticProps) {
      if (protoProps)
        defineProperties(Constructor.prototype, protoProps);
      if (staticProps)
        defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();
  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};
      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key))
            newObj[key] = obj[key];
        }
      }
      newObj["default"] = obj;
      return newObj;
    }
  }
  __name(_interopRequireWildcard, "_interopRequireWildcard");
  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : { "default": obj };
  }
  __name(_interopRequireDefault, "_interopRequireDefault");
  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }
  __name(_classCallCheck, "_classCallCheck");
  var _providersBasicprovider = basicprovider.exports;
  var _providersBasicprovider2 = _interopRequireDefault(_providersBasicprovider);
  var _providersServiceprovider = serviceprovider.exports;
  var _providersServiceprovider2 = _interopRequireDefault(_providersServiceprovider);
  var _providersFactoryprovider = factoryprovider.exports;
  var _providersFactoryprovider2 = _interopRequireDefault(_providersFactoryprovider);
  var _providersStaticprovider = staticprovider.exports;
  var _providersStaticprovider2 = _interopRequireDefault(_providersStaticprovider);
  var _helpersDecorator = decorator;
  var decoratorUtils = _interopRequireWildcard(_helpersDecorator);
  /*! Private definitions */
  var __containerId__ = Symbol();
  var __providerInstancesMap__ = Symbol();
  var __resourceInstancesMap__ = Symbol();
  var __providerConstructorsMap__ = Symbol();
  var __decoratorFunctionsMap__ = Symbol();
  /*! ========================================================================= */
  var Cation2 = function() {
    function Cation3() {
      var _ref = arguments.length <= 0 || arguments[0] === void 0 ? {} : arguments[0];
      var id = _ref.id;
      _classCallCheck(this, Cation3);
      this[__containerId__] = id;
      this[__providerInstancesMap__] = /* @__PURE__ */ new Map();
      this[__resourceInstancesMap__] = /* @__PURE__ */ new Map();
      this[__providerConstructorsMap__] = /* @__PURE__ */ new Map();
      this[__decoratorFunctionsMap__] = /* @__PURE__ */ new Map();
      this.addProvider("service", _providersServiceprovider2["default"]);
      this.addProvider("factory", _providersFactoryprovider2["default"]);
      this.addProvider("static", _providersStaticprovider2["default"]);
      this.register("container", this, {
        type: "static"
      });
    }
    __name(Cation3, "Cation");
    _createClass(Cation3, [{
      key: "getId",
      value: /* @__PURE__ */ __name(function getId() {
        return this[__containerId__];
      }, "getId")
    }, {
      key: "register",
      value: /* @__PURE__ */ __name(function register(id, resource) {
        var options2 = arguments.length <= 2 || arguments[2] === void 0 ? {} : arguments[2];
        if (!id) {
          throw new Error("`id` is required");
        }
        if (!resource) {
          throw new Error("`resource` is required");
        }
        if (this.has(id)) {
          throw new Error(`There's already a resource registered as "` + id + '"');
        }
        if (typeof options2.type === "undefined") {
          options2.type = "service";
        }
        if (typeof options2.args === "undefined") {
          options2.args = [];
        }
        if (typeof options2.decorators === "undefined") {
          options2.decorators = [];
        }
        if (typeof options2.tags === "undefined") {
          options2.tags = [];
        }
        if (!this.hasProvider(options2.type)) {
          throw new Error('Unknown type: "' + options2.type + '"');
        }
        var Provider = this[__providerConstructorsMap__].get(options2.type);
        this[__providerInstancesMap__].set(id, new Provider(this, id, resource, options2));
      }, "register")
    }, {
      key: "get",
      value: /* @__PURE__ */ __name(function get(id) {
        var _this = this;
        return new Promise(function(resolve, reject) {
          if (!_this.has(id)) {
            return reject(new Error('"' + id + '" resource not found'));
          }
          var provider = _this[__providerInstancesMap__].get(id);
          var isSingleton = provider.options.singleton;
          if (isSingleton && _this.isCached(id)) {
            return resolve(_this[__resourceInstancesMap__].get(id));
          }
          provider.get().then(function(resource) {
            var decoratorNames = provider.options.decorators;
            if (!decoratorNames.length) {
              return resource;
            }
            var decoratorFunctions = decoratorNames.map(function(name) {
              if (_this.hasDecorator(name)) {
                return _this[__decoratorFunctionsMap__].get(name);
              }
            });
            if (!decoratorFunctions.length) {
              return resource;
            }
            return decoratorFunctions.reduce(decoratorUtils.reducer, resource);
          }).then(function(resource) {
            if (isSingleton) {
              _this[__resourceInstancesMap__].set(id, resource);
            }
            return resource;
          }).then(function(resource) {
            return resolve(resource);
          })["catch"](function(error) {
            return reject(error);
          });
        });
      }, "get")
    }, {
      key: "has",
      value: /* @__PURE__ */ __name(function has(id) {
        if (this[__providerInstancesMap__].has(id)) {
          return true;
        }
        return false;
      }, "has")
    }, {
      key: "remove",
      value: /* @__PURE__ */ __name(function remove(id) {
        if (!this.has(id)) {
          return;
        }
        this[__providerInstancesMap__]["delete"](id);
      }, "remove")
    }, {
      key: "addProvider",
      value: /* @__PURE__ */ __name(function addProvider(name, providerFunction) {
        if (this.hasProvider(name)) {
          return;
        }
        var providerConstructorsMap = this[__providerConstructorsMap__];
        providerConstructorsMap.set(name, providerFunction);
      }, "addProvider")
    }, {
      key: "hasProvider",
      value: /* @__PURE__ */ __name(function hasProvider(name) {
        return this[__providerConstructorsMap__].has(name);
      }, "hasProvider")
    }, {
      key: "removeProvider",
      value: /* @__PURE__ */ __name(function removeProvider(name) {
        if (!this.hasProvider(name)) {
          return;
        }
        var providerConstructorsMap = this[__providerConstructorsMap__];
        providerConstructorsMap["delete"](name);
      }, "removeProvider")
    }, {
      key: "addDecorator",
      value: /* @__PURE__ */ __name(function addDecorator(name, decoratorFunction) {
        if (this.hasDecorator(name)) {
          return;
        }
        var decoratorFunctionsMap = this[__decoratorFunctionsMap__];
        decoratorFunctionsMap.set(name, decoratorFunction);
      }, "addDecorator")
    }, {
      key: "hasDecorator",
      value: /* @__PURE__ */ __name(function hasDecorator(name) {
        return this[__decoratorFunctionsMap__].has(name);
      }, "hasDecorator")
    }, {
      key: "removeDecorator",
      value: /* @__PURE__ */ __name(function removeDecorator(name) {
        if (!this.hasDecorator(name)) {
          return;
        }
        var decoratorFunctionsMap = this[__decoratorFunctionsMap__];
        decoratorFunctionsMap["delete"](name);
      }, "removeDecorator")
    }, {
      key: "isCached",
      value: /* @__PURE__ */ __name(function isCached(id) {
        return this[__resourceInstancesMap__].has(id);
      }, "isCached")
    }, {
      key: "clearCache",
      value: /* @__PURE__ */ __name(function clearCache() {
        this[__resourceInstancesMap__].clear();
      }, "clearCache")
    }, {
      key: "findTaggedResourceIds",
      value: /* @__PURE__ */ __name(function findTaggedResourceIds(tagName) {
        var providerInstancesMap = this[__providerInstancesMap__];
        var resourceIds = [];
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = void 0;
        try {
          for (var _iterator = providerInstancesMap.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _step$value = _slicedToArray(_step.value, 2);
            var resourceId = _step$value[0];
            var provider = _step$value[1];
            if (provider.options.tags.includes(tagName)) {
              resourceIds.push(resourceId);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
              _iterator["return"]();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
        return resourceIds;
      }, "findTaggedResourceIds")
    }]);
    return Cation3;
  }();
  exports["default"] = Cation2;
  exports.BasicProvider = _providersBasicprovider2["default"];
})(cation);
const Cation = /* @__PURE__ */ getDefaultExportFromCjs(cation);
const Container = /* @__PURE__ */ __name(() => {
  const container2 = new Cation();
  container2.addProvider("method", MethodProvider$1);
  container2.register("config", fetchConfig, {
    type: "method",
    singleton: true
  });
  container2.register("accessToken", getAccessToken, {
    args: ["@secret", "@expiration"],
    type: "method",
    singleton: true
  });
  container2.register("options", getOptions, {
    type: "method"
  });
  container2.register("secret", (config) => config.secret, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("expiration", (config) => {
    var _a;
    return (_a = config.accessTokenExpiration) != null ? _a : "30d";
  }, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("port", (config) => config.port, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("socketAddress", (config) => config.socketAddress, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("socketPort", (config) => config.socketPort, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("secure", (config) => config.secure, {
    args: ["@connectionConfig"],
    type: "method"
  });
  container2.register("machineConfig", (config) => config.machine, {
    args: ["@config"],
    type: "method"
  });
  container2.register("cncApi", CncApi, {
    args: ["@accessToken", "@socketAddress", "@socketPort", "@secure"],
    type: "method"
  });
  container2.register("ackBus", bus(), { type: "static" });
  container2.register("actionBus", bus(), { type: "static" });
  container2.register("connectionBus", bus(), { type: "static" });
  container2.register("connectionConfig", (config, options2) => {
    return {
      ...config.cncjs || {},
      ...options2 || {}
    };
  }, { args: ["@config", "@options"], type: "method", singleton: true });
  container2.register("socket", async (connection) => {
    const { socket } = await connection.connect();
    return socket;
  }, {
    args: ["@connection"],
    type: "method",
    singleton: true
  });
  container2.register("cncjsConfig", (config) => config.cncjs, {
    args: ["@config"],
    type: "method"
  });
  container2.register("connection", Connection$1, {
    args: ["@connectionConfig", "@accessToken"],
    singleton: true
  });
  container2.register("stateFeeder", StateFeeder, {
    args: ["@socket", "@ackBus"],
    type: "method",
    singleton: true
  });
  container2.register("cncActions", CncActions, {
    args: ["@socket", "@port", "@machineConfig", "@actionBus", "@ackBus"],
    type: "method",
    singleton: true
  });
  container2.register("buttonActions", ButtonActions, {
    args: ["@actionBus", "@connectionBus"],
    type: "method",
    singleton: true
  });
  return container2;
}, "Container");
const useScenesStore = pinia.defineStore("scenes", {
  state: () => ({
    _scenes: {}
  }),
  getters: {
    scenes: (state) => state._scenes
  },
  actions: {
    setScenes(scenes) {
      this._scenes = scenes;
    }
  }
});
const Bootstrap = /* @__PURE__ */ __name((container2) => {
  let stateFeeder;
  let cncActions;
  let buttonActions;
  const start = /* @__PURE__ */ __name(async () => {
    await initializeStores();
    return addConnectionListeners();
  }, "start");
  const addConnectionListeners = /* @__PURE__ */ __name(async () => {
    const connection = await container2.get("connection");
    const connectionBus = await container2.get("connectionBus");
    connectionBus.on("connect", () => {
      connection.openSerialPort();
    });
  }, "addConnectionListeners");
  const cleanup = /* @__PURE__ */ __name(async () => {
    const busPromises = ["actionBus", "ackBus", "connectionBus"].map((busName) => container2.get(busName));
    const busses = await Promise.all(busPromises);
    busses.forEach((bus2) => bus2.all.clear());
    stateFeeder == null ? void 0 : stateFeeder.destroy();
    buttonActions == null ? void 0 : buttonActions.destroy();
    container2.remove("stateFeeder");
    container2.remove("cncActions");
    container2.remove("buttonActions");
  }, "cleanup");
  const initializeStores = /* @__PURE__ */ __name(async () => {
    var _a, _b, _c, _d, _e, _f;
    const uiStore = useUiStore();
    const cncStore = useCncStore();
    const fileListStore = useFileListStore();
    const buttonStore = useButtonStore();
    const sceneStore = useScenesStore();
    const config = await container2.get("config");
    buttonStore.setButtons(Object.freeze(config.buttons));
    sceneStore.setScenes(Object.freeze(config.scenes));
    cncStore.setAxes((_a = config.machine) == null ? void 0 : _a.axes);
    uiStore.setScene("home");
    const uiConfig = { ...config.ui };
    {
      Object.assign(uiConfig, config.streamdeckUi || {});
    }
    uiStore.setBrightness(uiConfig.brightness);
    uiStore.setPalette(uiConfig.palette);
    uiStore.setGrid(uiConfig.rows, uiConfig.columns);
    uiStore.setThrottle(uiConfig.throttle);
    uiStore.setGcodeLimit(uiConfig.gcodeLimit);
    uiStore.textColor = (_b = uiConfig.textColor) != null ? _b : uiStore.textColor;
    uiStore.textShadow = uiConfig.textShadow;
    uiStore.font = (_c = uiConfig.font) != null ? _c : uiStore.font;
    uiStore.fontSize = (_d = uiConfig.fontSize) != null ? _d : uiStore.fontSize;
    uiStore.lineHeight = (_e = uiConfig.lineHeight) != null ? _e : uiStore.lineHeight;
    uiStore.pageColor = (_f = uiConfig.pageColor) != null ? _f : uiStore.pageColor;
    uiStore.setGcodeColors(uiConfig.gcodeColors);
    uiStore.setBgColor(uiConfig.bgColor);
    uiStore.setProgressColor(uiConfig.progressColor);
    await Promise.all([
      (async () => {
        const apiClient = await container2.get("cncApi");
        fileListStore.setClient(apiClient);
        cncStore.setClient(apiClient);
      })(),
      (async () => {
        const token = await container2.get("accessToken");
        cncStore.setToken(token);
      })()
    ]);
    const socket = await container2.get("socket");
    try {
      await socket.connect();
    } catch (e) {
      console.error("socket failed");
    }
    [stateFeeder, cncActions, buttonActions] = await Promise.all([
      container2.get("stateFeeder"),
      container2.get("cncActions"),
      container2.get("buttonActions")
    ]);
    return true;
  }, "initializeStores");
  return {
    cleanup,
    start
  };
}, "Bootstrap");
const arrayWrap = /* @__PURE__ */ __name((n) => {
  if (Array.isArray(n)) {
    return n;
  } else {
    return [n];
  }
}, "arrayWrap");
const useLoading = /* @__PURE__ */ __name((config) => {
  const cnc = useCncStore();
  const ids = vue.computed(() => {
    if (!config.actions) {
      return [];
    }
    return config.actions.reduce((list, action) => {
      var _a;
      if (action.action === "command" && action.arguments) {
        if (!cnc.commands) {
          return list;
        }
        let [id, name] = action.arguments;
        if (!id && name) {
          id = (_a = cnc.commands) == null ? void 0 : _a[name];
        }
        if (id) {
          list.push(id);
        }
      }
      return list;
    }, []);
  });
  const workerNames = ["outline"];
  const workerActions = vue.computed(() => {
    if (!config.actions) {
      return [];
    }
    return config.actions.map((action) => action.action).filter((action) => {
      return workerNames.includes(action);
    });
  });
  const loading = vue.computed(() => {
    return workerActions.value.some((id) => cnc.workerRunning(id)) || ids.value.some((id) => cnc.commandRunning(id));
  });
  return { loading };
}, "useLoading");
const Color = /* @__PURE__ */ __name(() => {
  const ui = useUiStore();
  const normalizeColor = /* @__PURE__ */ __name((color) => {
    if (typeof color === "number") {
      return ui.palette[color];
    }
    return color;
  }, "normalizeColor");
  const highlightColor = /* @__PURE__ */ __name((color) => {
    const amount = ui.isWeb ? 10 : 20;
    const normalized = tinycolor__default.default(normalizeColor(color));
    if (normalized.isDark()) {
      return normalized.brighten(amount).toHexString();
    } else {
      return normalized.darken(amount).toHexString();
    }
  }, "highlightColor");
  const contrastColor = /* @__PURE__ */ __name((color) => {
    return tinycolor__default.default(color).isLight() ? "#000" : "#fff";
  }, "contrastColor");
  return {
    highlightColor,
    normalizeColor,
    contrastColor
  };
}, "Color");
const splitText = /* @__PURE__ */ __name((text) => text.split(new RegExp("(?={{(?:.*?)}})|(?<={{(?:.*?)}})", "g")), "splitText");
const TextTemplate = /* @__PURE__ */ __name((text) => {
  const builder = splitText(text).map((component) => {
    if (/{{(.*?)}}/.test(component)) {
      const tokenList = evaluate.token(component.replace(/{{(.*)}}/, "$1"));
      const compiler = new evaluate.Compiler(tokenList);
      const ast = compiler.parse();
      return (context) => compiler.calc(ast, context);
    } else {
      return () => component;
    }
  });
  return (context) => builder.reduce((str, method) => str + method(context), "");
}, "TextTemplate");
const alignment = {
  "top left": { v: "top", h: "left" },
  "top center": { v: "top", h: "center" },
  "top right": { v: "top", h: "right" },
  left: { v: "center", h: "left" },
  center: { v: "center", h: "center" },
  right: { v: "center", h: "right" },
  "bottom left": { v: "bottom", h: "left" },
  "bottom center": { v: "bottom", h: "center" },
  "bottom right": { v: "bottom", h: "right" }
};
const svgAlignment = {
  "top left": { v: "text-after-edge", h: "start" },
  "top center": { v: "text-after-edge", h: "middle" },
  "top right": { v: "text-after-edge", h: "end" },
  left: { v: "middle", h: "start" },
  center: { v: "middle", h: "middle" },
  right: { v: "middle", h: "end" },
  "bottom left": { v: "text-before-edge", h: "start" },
  "bottom center": { v: "text-before-edge", h: "middle" },
  "bottom right": { v: "text-before-edge", h: "end" }
};
const useText = /* @__PURE__ */ __name((config) => {
  const cnc = useCncStore();
  const ui = useUiStore();
  const gcode = useGcodeStore();
  const scope = { cnc, ui, gcode };
  const color = Color();
  const textTemplate = vue.computed(() => TextTemplate(config.text));
  const textString = vue.computed(() => {
    return config.text && textTemplate.value(scope);
  });
  const textLines = vue.computed(() => {
    return config.text && textString.value.replace(/</g, "&lt;").replace(/>/g, "&gt;").split("\n");
  });
  const fontSize = vue.computed(() => {
    var _a;
    return `${(_a = config.textSize) != null ? _a : 1}em`;
  });
  const svgFontSize = vue.computed(() => {
    var _a;
    return ui.fontSize * ((_a = config.textSize) != null ? _a : 1);
  });
  const { lineHeight, font } = pinia.storeToRefs(ui);
  const cellTextColor = vue.computed(() => color.normalizeColor(ui.textColor));
  const contrastingTextColor = vue.computed(() => color.contrastColor(cellTextColor.value));
  const textAlignment = vue.computed(() => {
    var _a, _b;
    return (_b = (_a = alignment[config.textAlignment]) == null ? void 0 : _a.h) != null ? _b : "center";
  });
  const textVerticalAlignment = vue.computed(() => {
    var _a, _b;
    return (_b = (_a = alignment[config.textAlignment]) == null ? void 0 : _a.v) != null ? _b : "center";
  });
  const textSvgAlignment = vue.computed(() => {
    var _a, _b;
    return (_b = (_a = svgAlignment[config.textAlignment]) == null ? void 0 : _a.h) != null ? _b : "middle";
  });
  const textSvgVerticalAlignment = vue.computed(() => {
    var _a, _b;
    return (_b = (_a = svgAlignment[config.textAlignment]) == null ? void 0 : _a.v) != null ? _b : "middle";
  });
  return {
    cellTextColor,
    contrastingTextColor,
    font,
    fontSize,
    lineHeight,
    svgFontSize,
    textAlignment,
    textLines,
    textString,
    textSvgAlignment,
    textSvgVerticalAlignment,
    textVerticalAlignment
  };
}, "useText");
const useEvaluate = /* @__PURE__ */ __name(() => {
  const cnc = useCncStore();
  const ui = useUiStore();
  const gcode = useGcodeStore();
  const scopedEvaluate = /* @__PURE__ */ __name((str) => evaluate__default.default({ cnc, ui, gcode }, str), "scopedEvaluate");
  return { scopedEvaluate };
}, "useEvaluate");
const watchCompute = /* @__PURE__ */ __name((getter) => {
  const computedValue = vue.computed(getter);
  const refValue = vue.ref();
  vue.watch(computedValue, (newValue) => {
    refValue.value = newValue;
  }, { immediate: true });
  return refValue;
}, "watchCompute");
const useVisibility = /* @__PURE__ */ __name((config, buttonActions) => {
  const { scopedEvaluate } = useEvaluate();
  const show = watchCompute(() => !config.if || !!scopedEvaluate(config.if));
  const configDisabled = vue.computed(() => config.disabled && !!scopedEvaluate(config.disabled));
  const enabled = watchCompute(() => {
    var _a;
    return !configDisabled.value && ((_a = buttonActions.value) == null ? void 0 : _a.enabled(config.actions));
  });
  return { enabled, show };
}, "useVisibility");
const useButton = /* @__PURE__ */ __name((events, cleanupActions) => {
  const active = vue.ref(false);
  const holding = vue.ref(false);
  let skipUpEvent = false;
  const setActive = /* @__PURE__ */ __name(() => {
    active.value = true;
  }, "setActive");
  const unsetActive = /* @__PURE__ */ __name(() => {
    active.value = false;
  }, "unsetActive");
  const setHold = /* @__PURE__ */ __name(() => {
    holding.value = true;
    skipUpEvent = false;
  }, "setHold");
  const unsetHold = /* @__PURE__ */ __name(() => {
    holding.value = false;
    skipUpEvent = false;
  }, "unsetHold");
  const onHold = /* @__PURE__ */ __name(() => {
    unsetActive();
    holding.value = false;
    skipUpEvent = true;
    callActions(configHoldActions.value);
  }, "onHold");
  const configDownActions = vue.computed(() => {
    var _a;
    return (_a = events.value.down) != null ? _a : [];
  });
  const configUpActions = vue.computed(() => {
    var _a;
    return (_a = events.value.up) != null ? _a : [];
  });
  const configHoldActions = vue.computed(() => {
    var _a;
    return (_a = events.value.hold) != null ? _a : [];
  });
  const hasHoldAction = vue.computed(() => configHoldActions.value.length > 0);
  const callActions = /* @__PURE__ */ __name((actionSet) => {
    actionSet.forEach((config) => {
      config.action(...config.arguments || []);
    });
  }, "callActions");
  const downHandler = /* @__PURE__ */ __name(() => {
    if (!active.value) {
      setActive();
      callActions(configDownActions.value);
    }
  }, "downHandler");
  const upHandler = /* @__PURE__ */ __name(() => {
    const wasActive = active.value;
    unsetActive();
    if (skipUpEvent) {
      skipUpEvent = false;
      return;
    }
    if (wasActive) {
      callActions(configUpActions.value);
    }
  }, "upHandler");
  const hasButton = vue.computed(() => {
    return Object.keys(events.value).length > 0;
  });
  const cancelClick = /* @__PURE__ */ __name(() => {
    const wasActive = active.value;
    unsetActive();
    if (wasActive && cleanupActions.value) {
      callActions(cleanupActions.value);
    }
  }, "cancelClick");
  const clearActivity = /* @__PURE__ */ __name(() => {
    unsetHold();
    if (!active.value) {
      return;
    }
    if (cleanupActions.value) {
      callActions(cleanupActions.value);
    }
    unsetActive();
  }, "clearActivity");
  return {
    active,
    cancelClick,
    clearActivity,
    downHandler,
    hasButton,
    hasHoldAction,
    holding,
    onHold,
    setActive,
    setHold,
    unsetHold,
    upHandler
  };
}, "useButton");
class ButtonHandler {
  constructor(actions, buttonActions) {
    this.buttonActions = buttonActions;
    this.actions = actions;
    this.setup();
  }
  down(index) {
    if (!this.hasButton.value) {
      return;
    }
    if (this.active.value) {
      return;
    }
    this.buttons.add(index);
    this.downHandler();
    if (this.hasHoldAction.value) {
      this.setHold();
      this.holdTimer = setTimeout(() => {
        this.onHold();
      }, 500);
    }
  }
  up(index) {
    if (!this.hasButton.value) {
      return;
    }
    if (!this.active.value || !this.buttons.has(index)) {
      return;
    }
    this.unsetHold();
    this.buttons.delete(index);
    clearTimeout(this.holdTimer);
    this.upHandler();
  }
  cleanup() {
    clearTimeout(this.holdTimer);
    this.buttons.clear();
    this.clearActivity();
  }
  setup() {
    this.buttons = /* @__PURE__ */ new Set();
    const events = vue.computed(() => {
      var _a;
      return (_a = this.buttonActions.value) == null ? void 0 : _a.getHandlers(this.actions);
    });
    const cleanupActions = vue.computed(() => {
      var _a;
      return (_a = this.buttonActions.value) == null ? void 0 : _a.ensureHandler(this.actions);
    });
    const helpers = useButton(events, cleanupActions);
    Object.entries(helpers).forEach(([name, method]) => {
      this[name] = method;
    });
  }
}
__name(ButtonHandler, "ButtonHandler");
const icons = {};
const inputSize = 144;
const textOffset = /* @__PURE__ */ __name((alignment2, width) => {
  switch (alignment2) {
    case "start":
      return 4;
    case "middle":
      return width / 2;
    case "end":
      return width - 4;
  }
}, "textOffset");
const textVOffset = /* @__PURE__ */ __name((alignment2, height, textHeight) => {
  switch (alignment2) {
    case "text-after-edge":
      return 4;
    case "middle":
      return (height - textHeight) / 2;
    case "text-before-edge":
      return height - textHeight - 4;
  }
}, "textVOffset");
const getRender = /* @__PURE__ */ __name(async (config, canvas2, directory) => {
  var _a, _b, _c, _d;
  if (!config.show) {
    return [];
  }
  const columns = (_a = config.columns) != null ? _a : 1;
  const rows = (_b = config.rows) != null ? _b : 1;
  const outputSize = (_c = config.buttonSize) != null ? _c : 72;
  const inputWidth = columns * inputSize;
  const inputHeight = rows * inputSize;
  const outputWidth = config.width;
  const outputHeight = config.height;
  const createConfig = { width: inputWidth, height: inputHeight, channels: 4 };
  if (config.color) {
    createConfig.background = config.color;
  }
  let image = sharp__default.default({
    create: createConfig
  });
  const composite = [];
  const finalComposite = [];
  if (config.icon) {
    if (!icons[config.icon]) {
      const overlay = await sharp__default.default(path__default.default.resolve(directory, config.icon)).resize({
        width: inputWidth,
        height: inputHeight,
        fit: "contain",
        kernel: "cubic",
        background: { r: 0, g: 0, b: 0, alpha: 0 }
      }).toBuffer();
      icons[config.icon] = overlay;
    }
    composite.push({
      input: icons[config.icon]
    });
  }
  if (config.type === "gcodePreview") {
    const { width, height } = canvas2.dimensions();
    finalComposite.push({
      input: await canvas2.buffer(),
      top: Math.floor((outputHeight - height) / 2),
      left: Math.floor((outputWidth - width) / 2)
    });
  }
  if (config.text) {
    const fontSize = config.fontSize;
    const lineHeight = ((_d = config.lineHeight) != null ? _d : 1.1) * fontSize;
    const spanX = textOffset(config.textSvgAlignment, outputWidth);
    const textSpan = config.textLines.map((line) => {
      const text = line === "" ? " " : line;
      return `<tspan dy="${lineHeight}" x="${spanX}px">${escape__default.default(text)}</tspan>`;
    });
    const y = textVOffset(config.textSvgVerticalAlignment, outputHeight, config.textLines.length * lineHeight);
    const svg = `<svg width="${outputWidth}px" height="${outputHeight}px">
      <text
        fill="${config.cellTextColor}"
        font-size="${fontSize}px"
        font-family="${config.font}"
        text-anchor="${config.textSvgAlignment}"
        dominant-baseline="${config.textSvgVerticalAlignment}"
        y="${y}"
        x="0">${textSpan.join("")}</text>
    </svg>`;
    const textBuffer = Buffer.from(svg);
    finalComposite.push({ input: textBuffer, top: 0, left: 0 });
  }
  if (config.holdPercent) {
    const height = outputHeight * config.holdPercent * 1.1;
    const svg = `<svg width="${outputWidth}px" height="${outputHeight}px">
      <rect
      width="${outputWidth}px"
      height="${height}px"
      x="0"
      y="${outputHeight - height}px"
      stroke="none"
      fill="${config.cellProgressColor}" />
    </svg>`;
    const holdBuffer = Buffer.from(svg);
    finalComposite.push({ input: holdBuffer, top: 0, left: 0 });
  }
  if (config.loadingPercent != null) {
    const radius = Math.min(outputHeight, outputWidth) * 0.4;
    const circ = radius * 2 * Math.PI;
    const rotation = 360 * config.loadingPercent * 3;
    const dashOffset = (config.loadingPercent + 0.5) % 1 * 2 - 1;
    const svg = `<svg width="${outputWidth}px" height="${outputHeight}px">
      <circle
      stroke="${config.cellProgressColor}"
      fill="none"
      stroke-width="8px"
      stroke-linecap="round"
      stroke-dasharray="${circ}"
      stroke-dashoffset="${circ - dashOffset * circ}"
      transform="rotate(${rotation} ${outputWidth / 2} ${outputHeight / 2})"
      cx="${outputWidth / 2}"
      cy="${outputHeight / 2}"
      r="${radius}" />
    </svg>`;
    const holdBuffer = Buffer.from(svg);
    finalComposite.push({ input: holdBuffer, top: 0, left: 0 });
  }
  if (!config.enabled) {
    finalComposite.push({
      input: Buffer.from([0, 0, 0, 180]),
      raw: { width: 1, height: 1, channels: 4 },
      tile: true
    });
  }
  if (composite.length) {
    image = sharp__default.default(await image.composite(composite).raw().toBuffer(), {
      raw: { width: inputWidth, height: inputHeight, channels: 4 }
    });
  }
  image = image.resize({
    width: outputWidth,
    height: outputHeight,
    kernel: "cubic"
  }).composite(finalComposite);
  if (columns > 1 || rows > 1) {
    const grid = [];
    image = sharp__default.default(await image.raw().toBuffer(), {
      raw: { width: outputWidth, height: outputHeight, channels: 4 }
    });
    for (let currentRow = 0; currentRow < rows; currentRow++) {
      for (let currentColumn = 0; currentColumn < columns; currentColumn++) {
        grid.push(await image.extract({
          left: currentColumn * outputSize,
          top: currentRow * outputSize,
          width: outputSize,
          height: outputSize
        }).raw().toBuffer());
      }
    }
    return Promise.all(grid);
  }
  const outputBuffer = await image.raw().toBuffer();
  return [outputBuffer];
}, "getRender");
const buttonRenderer = getRender;
let canvas;
let isNodeCanvas = false;
let isPureImage = false;
try {
  canvas = require("canvas");
  isNodeCanvas = true;
} catch (e) {
}
if (!canvas) {
  try {
    canvas = require("pureimage");
    isPureImage = {};
  } catch (e) {
    canvas = {};
  }
}
class nodeCanvas {
  constructor(width, height) {
    this.width = width;
    this.height = height;
    this.canvas = canvas.createCanvas(width, height);
  }
  dimensions() {
    return { width: this.canvas.width, height: this.canvas.height };
  }
  buffer() {
    return this.canvas.toBuffer();
  }
}
__name(nodeCanvas, "nodeCanvas");
class pureImage {
  constructor(width, height, scale = 2) {
    this.width = width;
    this.height = height;
    this.scale = scale;
    this.lineWidth = 1.01;
    this.canvas = canvas.make(width * this.scale, height * this.scale);
  }
  dimensions() {
    return {
      width: this.canvas.width / this.scale,
      height: this.canvas.height / this.scale
    };
  }
  async buffer() {
    const PassThrough = require("stream").PassThrough;
    const passThroughStream = new PassThrough();
    const pngData = [];
    passThroughStream.on("data", (chunk) => pngData.push(chunk));
    passThroughStream.on("end", () => {
    });
    await canvas.encodePNGToStream(this.canvas, passThroughStream);
    await canvas.encodePNGToStream(this.canvas, fs__default.default.createWriteStream("canvas.png"));
    return sharp__default.default(Buffer.concat(pngData)).resize({
      width: this.width,
      height: this.height,
      fit: "contain",
      kernel: "cubic"
    }).toBuffer();
  }
}
__name(pureImage, "pureImage");
class noop {
  constructor(width, height) {
    this.width = width;
    this.height = height;
    console.warn("Requested a new canvas but pureimage or node-canvas are not installed");
  }
  buffer() {
    return null;
  }
}
__name(noop, "noop");
let CanvasImplementation = noop;
if (isNodeCanvas) {
  CanvasImplementation = nodeCanvas;
} else if (isPureImage) {
  CanvasImplementation = pureImage;
}
const Canvas = CanvasImplementation;
const uncompute = /* @__PURE__ */ __name((computed) => Object.entries(computed).reduce((obj, [name, computed2]) => {
  obj[name] = computed2.value;
  return obj;
}, {}), "uncompute");
const useColor = /* @__PURE__ */ __name((config) => {
  const ui = useUiStore();
  const color = Color();
  const cellBgColor = vue.computed(() => color.normalizeColor(config.bgColor || ui.bgColor));
  const cellProgressColor = vue.computed(() => color.normalizeColor(ui.progressColor));
  const cellActiveColor = vue.computed(() => color.highlightColor(cellBgColor.value));
  return { cellBgColor, cellProgressColor, cellActiveColor };
}, "useColor");
const useGcode = /* @__PURE__ */ __name((config) => {
  const gcode = useGcodeStore();
  const ui = useUiStore();
  const renderGcode = vue.computed(() => config.type === "gcodePreview" && gcode.geometry);
  const { gcodeColors } = pinia.storeToRefs(ui);
  return { renderGcode, gcodeColors };
}, "useGcode");
function calculateRatio(size, canvas2) {
  var pW = Math.abs(size.max.x - size.min.x);
  var pH = Math.abs(size.max.y - size.min.y);
  var cW = parseInt(canvas2.width, 10), cH = parseInt(canvas2.height, 10);
  return Math.min(cW / pW, cH / pH);
}
__name(calculateRatio, "calculateRatio");
function drawStraightLine(ctx, ratio, start, line, height, color, settings = {}) {
  const startX = ratio * (line.start.x - start.x);
  const startY = height - ratio * (line.start.y - start.y);
  const endX = ratio * (line.end.x - start.x);
  const endY = height - ratio * (line.end.y - start.y);
  if (startX === endX && startY === endY) {
    return;
  }
  ctx.beginPath();
  ctx.moveTo(startX, startY);
  ctx.lineTo(endX, endY);
  ctx.strokeStyle = color;
  ctx.lineWidth = settings.lineWidth;
  ctx.stroke();
  ctx.closePath();
}
__name(drawStraightLine, "drawStraightLine");
function drawCurvedLine(ctx, ratio, start, line, height, color, settings = {}) {
  var i = 0;
  var b = line.beziers, l = {};
  ctx.beginPath();
  for (i = 0; i < b.length; i++) {
    l = b[i];
    ctx.moveTo(ratio * (l.p0.x - start.x), height - ratio * (l.p0.y - start.y));
    ctx.bezierCurveTo(ratio * (l.p1.x - start.x), height - ratio * (l.p1.y - start.y), ratio * (l.p2.x - start.x), height - ratio * (l.p2.y - start.y), ratio * (l.p3.x - start.x), height - ratio * (l.p3.y - start.y));
  }
  ctx.strokeStyle = color;
  ctx.lineWidth = settings.lineWidth || 2;
  ctx.stroke();
  ctx.closePath();
}
__name(drawCurvedLine, "drawCurvedLine");
function preview(size, colors, settings, canvas2) {
  var _a;
  if (colors === void 0) {
    return;
  }
  settings != null ? settings : settings = {};
  if (size.max.x === size.min.x && size.max.y === size.min.y) {
    return;
  }
  let targetDimensions = {};
  if (settings.width && settings.height) {
    targetDimensions.width = settings.width;
    targetDimensions.height = settings.height;
  } else {
    targetDimensions.width = canvas2.width;
    targetDimensions.height = canvas2.height;
  }
  const ratio = calculateRatio(size, targetDimensions);
  const minLineWidth = (_a = settings.lineWidth) != null ? _a : 0.08;
  const lineWidth = Math.max(minLineWidth, ratio * 3e-3);
  const drawSettings = {
    lineWidth
  };
  const imageWidth = Math.abs(size.max.x - size.min.x);
  const imageHeight = Math.abs(size.max.y - size.min.y);
  if (settings.autosize) {
    canvas2.height = imageHeight * ratio + lineWidth;
    canvas2.width = imageWidth * ratio + lineWidth;
  }
  const start = {
    x: size.min.x - (canvas2.width / ratio - imageWidth) / 2,
    y: size.min.y - (canvas2.height / ratio - imageHeight) / 2
  };
  const cH = parseInt(canvas2.height, 10);
  const ctx = canvas2.getContext("2d");
  ctx.imageSmoothingEnabled = true;
  ctx.clearRect(0, 0, canvas2.width, canvas2.height);
  ctx.beginPath();
  ctx.rect(0, 0, canvas2.width, canvas2.height);
  ctx.fillStyle = settings.bg || "transparent";
  ctx.fill();
  ctx.imageSmoothingEnabled = true;
  ctx.lineCap = "round";
  return (line) => {
    if (line.type === "G0" && colors.G0 !== void 0) {
      drawStraightLine(ctx, ratio, start, line, cH, colors.G0, drawSettings);
    } else if (line.type === "G1" && colors.G1 !== void 0) {
      drawStraightLine(ctx, ratio, start, line, cH, colors.G1, drawSettings);
    } else if ((line.type === "G2" || line.type === "G3") && colors.G2G3 !== void 0) {
      drawCurvedLine(ctx, ratio, start, line, cH, colors.G2G3, drawSettings);
    }
  };
}
__name(preview, "preview");
const animatedWait = /* @__PURE__ */ __name((delay) => () => {
  return new Promise((resolve) => setTimeout(resolve, delay));
}, "animatedWait");
const defaultColors = {
  G0: "#3c7e30",
  G1: "#9a96ff",
  G2G3: "#9a96ff"
};
const renderToolpath = /* @__PURE__ */ __name((canvas2, parsedGcode, settings, callback = () => {
}, halted = () => false) => {
  const colors = { ...defaultColors, ...settings.colors || {} };
  const options2 = { autosize: true, ...settings };
  const drawLine = preview(parsedGcode.size, colors, options2, canvas2);
  if (!drawLine) {
    return;
  }
  if (options2.animate) {
    return animatedDraw({
      lines: parsedGcode.lines,
      drawLine,
      duration: 4e3,
      throttle: settings.throttle,
      callback,
      halted
    });
  } else {
    return draw(parsedGcode.lines, drawLine, callback, halted);
  }
}, "renderToolpath");
function* chunkLines(lines, size = 1) {
  for (let i = 0, l = lines.length; i < l; i += size) {
    yield [i + size, lines.slice(i, i + size)];
  }
}
__name(chunkLines, "chunkLines");
const animatedDraw = /* @__PURE__ */ __name(async ({
  lines,
  drawLine,
  duration,
  callback,
  halted,
  throttle: throttle2
}) => {
  let fps = 30;
  if (throttle2) {
    fps = 1e3 / throttle2;
  }
  const linesPerFrame = Math.max(1, Math.floor(lines.length / (duration / 1e3 * fps)));
  const wait = animatedWait(1e3 / fps);
  const gen = chunkLines(lines, linesPerFrame);
  for (const [index, chunk] of gen) {
    if (halted()) {
      return;
    }
    chunk.forEach(drawLine);
    callback(index);
    await wait();
  }
  callback(lines.length);
}, "animatedDraw");
const draw = /* @__PURE__ */ __name(async (lines, drawLine, callback, halted) => {
  let nextWait = Date.now();
  const wait = animatedWait(60);
  for (const [index, line] of Object.entries(lines)) {
    if (halted()) {
      return;
    }
    drawLine(line);
    if (Date.now() > nextWait) {
      callback(index);
      await wait();
      nextWait = Date.now() + 50;
    }
  }
  callback(lines.length);
}, "draw");
class CliButton {
  constructor(index, config, { size, buttonActions, iconDirectory, throttle: throttle2 }) {
    var _a, _b;
    this.index = index;
    this.config = config;
    this.buffers = Array.from(Array(((_a = config.rows) != null ? _a : 1) * ((_b = config.columns) != null ? _b : 1))).map(() => vue.ref());
    this.size = size;
    this.buttonActions = buttonActions;
    this.iconDirectory = iconDirectory;
    this.watchers = [];
    this.drawTime = 0;
    this.throttle = throttle2;
    this.setup();
  }
  down() {
    if (this.enabled.value) {
      this.buttonHandler.down();
    }
  }
  up() {
    if (this.enabled.value) {
      this.buttonHandler.up();
    }
  }
  cleanup() {
    this.buttonHandler.cleanup();
    this.watchers.forEach((watcher) => {
      watcher();
    });
  }
  watch(key, callback, options2 = {}) {
    this.watchers.push(vue.watch(key, callback, options2));
  }
  watchEffect(callback) {
    this.watchers.push(vue.watchEffect(callback));
  }
  setup() {
    var _a, _b;
    const { cellBgColor, cellProgressColor, cellActiveColor } = useColor(this.config);
    const {
      cellTextColor,
      contrastingTextColor,
      font,
      lineHeight,
      svgFontSize: fontSize,
      textSvgAlignment,
      textSvgVerticalAlignment,
      textLines
    } = useText(this.config);
    const width = ((_a = this.config.columns) != null ? _a : 1) * this.size;
    const height = ((_b = this.config.rows) != null ? _b : 1) * this.size;
    this.buttonHandler = new ButtonHandler(this.config.actions, this.buttonActions);
    const { renderGcode, gcodeColors } = useGcode(this.config);
    const { show, enabled } = useVisibility(this.config, this.buttonActions);
    const { loading } = useLoading(this.config);
    this.show = show;
    this.enabled = enabled;
    const color = vue.computed(() => {
      let color2;
      if (this.enabled.value) {
        if (this.buttonHandler.active.value) {
          color2 = cellActiveColor.value;
        } else {
          color2 = cellBgColor.value;
        }
      }
      return color2 != null ? color2 : "#000";
    });
    const holdPercent = vue.ref(0);
    let holdAnimation;
    this.watch(this.buttonHandler.holding, (holding) => {
      if (holdAnimation) {
        holdAnimation.cancel();
        holdAnimation = null;
        holdPercent.value = 0;
      }
      if (holding) {
        holdAnimation = animation({
          duration: 400,
          fps: 60,
          callback: (percent) => {
            holdPercent.value = percent;
          }
        });
        holdAnimation.delay(100);
      }
    });
    const loadingPercent = vue.ref(null);
    let loadingAnimation;
    this.watch(loading, (current) => {
      if (loadingAnimation) {
        loadingAnimation.cancel();
        loadingAnimation = null;
        loadingPercent.value = null;
      }
      if (current) {
        loadingAnimation = animation({
          duration: 5e3,
          fps: 60,
          loop: true,
          callback: (percent) => {
            loadingPercent.value = percent;
          }
        });
        loadingAnimation.start();
      }
    }, { immediate: !!loading.value });
    const gcodeLine = vue.ref();
    const updateGcodeLine = /* @__PURE__ */ __name((index) => {
      gcodeLine.value = index;
    }, "updateGcodeLine");
    if (this.config.type === "gcodePreview") {
      this.watchEffect((onInvalidate) => {
        this.canvas = new Canvas(width - 10, height - 10);
        if (!renderGcode.value) {
          return;
        }
        let halt = false;
        onInvalidate(() => {
          halt = true;
        });
        const halted = /* @__PURE__ */ __name(() => halt, "halted");
        const settings = {
          animate: this.config.animated,
          throttle: this.throttle,
          lineWidth: this.canvas.lineWidth,
          autosize: false,
          colors: gcodeColors.value
        };
        renderToolpath(this.canvas.canvas, renderGcode.value, settings, updateGcodeLine, halted);
      });
    }
    this.show = show;
    const render = /* @__PURE__ */ __name((time, config) => {
      buttonRenderer(config, this.canvas, this.iconDirectory).then((newBuffers) => {
        if (time < this.drawTime) {
          return;
        }
        this.drawTime = time;
        this.buffers.forEach((buffer, i) => {
          buffer.value = newBuffers[i] || null;
        });
      });
    }, "render");
    const throttledRender = this.throttle ? throttle__default.default(render, this.throttle) : render;
    this.watchEffect(() => {
      const time = performance.now();
      const config = {
        ...this.config,
        buttonSize: this.size,
        height,
        index: this.index,
        width,
        ...uncompute({
          color,
          cellProgressColor,
          cellTextColor,
          contrastingTextColor,
          enabled,
          lineHeight,
          font,
          fontSize,
          gcodeLine,
          holdPercent,
          loadingPercent,
          renderGcode,
          textSvgAlignment,
          textSvgVerticalAlignment,
          textLines,
          show
        })
      };
      throttledRender(time, config);
    });
  }
}
__name(CliButton, "CliButton");
const container = Container();
pinia.setActivePinia(pinia.createPinia());
const bootstrap = Bootstrap(container);
const getStreamdeck = /* @__PURE__ */ __name(async () => {
  let streamdeck;
  try {
    streamdeck = await node.openStreamDeck(null, {
      jpegOptions: { quality: 100, subsampling: 0 }
    });
  } catch (e) {
    console.error(e);
    console.error("Could not open Stream Deck");
    process.exit(1);
  }
  return streamdeck;
}, "getStreamdeck");
const run = /* @__PURE__ */ __name(async () => {
  const { directory } = await container.get("options");
  let streamdeck;
  try {
    [, streamdeck] = await Promise.all([bootstrap.start(), getStreamdeck()]);
  } catch (e) {
    console.error("Connection error", e);
    process.kill(process.pid, "SIGINT");
  }
  const { buttons: buttonConfig } = useButtonStore();
  const sceneStore = useScenesStore();
  const ui = useUiStore();
  ui.setWeb(false);
  ui.setIconSize(streamdeck.ICON_SIZE);
  const { ui: uiConfig } = await container.get("config");
  const iconDirectory = path__default.default.join(directory, "icons");
  const { wake } = SleepScreen(uiConfig == null ? void 0 : uiConfig.timeout, streamdeck);
  const { buttons: fileListButtons, loadFiles } = useFileList();
  const renderBuffers = Array.from(Array(ui.rows * ui.columns)).map(() => vue.ref());
  renderBuffers.forEach((buffer, index) => {
    vue.watchEffect(() => {
      const result = buffer.value;
      if (result) {
        streamdeck.fillKeyBuffer(index, result, { format: "rgba" });
      } else {
        streamdeck.clearKey(index);
      }
    });
  });
  const specialScenes = {
    gcodeList: {
      buttons: fileListButtons,
      load: loadFiles
    }
  };
  const buttons = vue.computed(() => {
    var _a;
    const specialScene = specialScenes[ui.sceneName];
    if (specialScene) {
      (_a = specialScene.load) == null ? void 0 : _a.call(specialScene);
      return specialScene.buttons.value;
    } else {
      return sceneStore.scenes[ui.sceneName].buttons;
    }
  });
  const buttonActions = vue.ref();
  container.get("buttonActions").then((actions) => {
    buttonActions.value = actions;
  });
  const eachButton = /* @__PURE__ */ __name((callback) => {
    buttons.value.forEach((row, r) => {
      row.forEach((position, c) => {
        const key = r * ui.columns + c;
        arrayWrap(position).forEach((buttonId) => {
          if (buttonId == null) {
            return;
          }
          callback(key, buttonId);
        });
      });
    });
  }, "eachButton");
  const sceneButtons = vue.ref({});
  vue.watch(buttons, () => {
    const buttonList = {};
    eachButton((key, buttonId) => {
      var _a, _b, _c, _d;
      const config = typeof buttonId === "string" ? buttonConfig[buttonId] : buttonId;
      const button = new CliButton(key, config, {
        size: streamdeck.ICON_SIZE,
        buttonActions,
        iconDirectory,
        throttle: ui.throttle
      });
      for (let subR = 0; subR < ((_a = config.rows) != null ? _a : 1); subR++) {
        for (let subC = 0; subC < ((_b = config.columns) != null ? _b : 1); subC++) {
          const subOffset = subC + subR * ((_c = config.columns) != null ? _c : 1);
          const globalKey = key + subR * ui.columns + subC;
          (_d = buttonList[globalKey]) != null ? _d : buttonList[globalKey] = [];
          buttonList[globalKey].push({
            row: subR,
            column: subC,
            offset: subOffset,
            button
          });
        }
      }
    });
    sceneButtons.value = Object.freeze(buttonList);
  }, { immediate: true });
  const effectiveButtons = vue.computed(() => {
    return Object.entries(sceneButtons.value).reduce((buttonList, [index, buttons2]) => {
      buttonList[index] = buttons2.slice().reverse().find((buttonPosition) => buttonPosition.button.show.value);
      return buttonList;
    }, {});
  });
  vue.watch(sceneButtons, (current, previous) => {
    Object.values(previous).forEach((position) => {
      position.forEach((button) => {
        var _a;
        (_a = button.button) == null ? void 0 : _a.cleanup();
      });
    });
  });
  renderBuffers.forEach((buffer, index) => {
    vue.watchEffect(() => {
      var _a, _b, _c, _d, _e;
      const button = effectiveButtons.value[index];
      const newBuffer = (_c = (_b = (_a = button == null ? void 0 : button.button) == null ? void 0 : _a.buffers) == null ? void 0 : _b[button == null ? void 0 : button.offset]) == null ? void 0 : _c.value;
      if (!buffer.value || !((_e = (_d = button == null ? void 0 : button.button) == null ? void 0 : _d.show) == null ? void 0 : _e.value) || newBuffer && !newBuffer.equals(buffer.value)) {
        buffer.value = newBuffer;
      }
    });
  });
  vue.watchEffect(() => {
    streamdeck.setBrightness(ui.displayBrightness);
  });
  streamdeck.on("down", (keyIndex) => {
    if (wake()) {
      return;
    }
    const button = effectiveButtons.value[keyIndex];
    if (!button) {
      return;
    }
    button.button.down();
  });
  streamdeck.on("up", (keyIndex) => {
    if (wake()) {
      return;
    }
    const button = effectiveButtons.value[keyIndex];
    if (!button) {
      return;
    }
    button.button.up();
    ui.activity();
  });
}, "run");
run();
