"use strict";
var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
const gcodetogeometry = require("./assets/gcodetogeometry.445d29d0.js");
const hull = require("hull.js");
const _interopDefaultLegacy = /* @__PURE__ */ __name((e) => e && typeof e === "object" && "default" in e ? e : { default: e }, "_interopDefaultLegacy");
const hull__default = /* @__PURE__ */ _interopDefaultLegacy(hull);
function distance(p1, p2) {
  return Math.sqrt(distanceSq(p1, p2));
}
__name(distance, "distance");
function distanceSq(p1, p2) {
  return Math.pow(p1[0] - p2[0], 2) + Math.pow(p1[1] - p2[1], 2);
}
__name(distanceSq, "distanceSq");
function distanceToSegmentSq(p, v, w) {
  const l2 = distanceSq(v, w);
  if (l2 === 0) {
    return distanceSq(p, v);
  }
  let t = ((p[0] - v[0]) * (w[0] - v[0]) + (p[1] - v[1]) * (w[1] - v[1])) / l2;
  t = Math.max(0, Math.min(1, t));
  return distanceSq(p, lerp(v, w, t));
}
__name(distanceToSegmentSq, "distanceToSegmentSq");
function lerp(a, b, t) {
  return [
    a[0] + (b[0] - a[0]) * t,
    a[1] + (b[1] - a[1]) * t
  ];
}
__name(lerp, "lerp");
function flatness(points, offset) {
  const p1 = points[offset + 0];
  const p2 = points[offset + 1];
  const p3 = points[offset + 2];
  const p4 = points[offset + 3];
  let ux = 3 * p2[0] - 2 * p1[0] - p4[0];
  ux *= ux;
  let uy = 3 * p2[1] - 2 * p1[1] - p4[1];
  uy *= uy;
  let vx = 3 * p3[0] - 2 * p4[0] - p1[0];
  vx *= vx;
  let vy = 3 * p3[1] - 2 * p4[1] - p1[1];
  vy *= vy;
  if (ux < vx) {
    ux = vx;
  }
  if (uy < vy) {
    uy = vy;
  }
  return ux + uy;
}
__name(flatness, "flatness");
function getPointsOnBezierCurveWithSplitting(points, offset, tolerance, newPoints) {
  const outPoints = newPoints || [];
  if (flatness(points, offset) < tolerance) {
    const p0 = points[offset + 0];
    if (outPoints.length) {
      const d = distance(outPoints[outPoints.length - 1], p0);
      if (d > 1) {
        outPoints.push(p0);
      }
    } else {
      outPoints.push(p0);
    }
    outPoints.push(points[offset + 3]);
  } else {
    const t = 0.5;
    const p1 = points[offset + 0];
    const p2 = points[offset + 1];
    const p3 = points[offset + 2];
    const p4 = points[offset + 3];
    const q1 = lerp(p1, p2, t);
    const q2 = lerp(p2, p3, t);
    const q3 = lerp(p3, p4, t);
    const r1 = lerp(q1, q2, t);
    const r2 = lerp(q2, q3, t);
    const red = lerp(r1, r2, t);
    getPointsOnBezierCurveWithSplitting([p1, q1, r1, red], 0, tolerance, outPoints);
    getPointsOnBezierCurveWithSplitting([red, r2, q3, p4], 0, tolerance, outPoints);
  }
  return outPoints;
}
__name(getPointsOnBezierCurveWithSplitting, "getPointsOnBezierCurveWithSplitting");
function simplifyPoints(points, start, end, epsilon, newPoints) {
  const outPoints = newPoints || [];
  const s = points[start];
  const e = points[end - 1];
  let maxDistSq = 0;
  let maxNdx = 1;
  for (let i = start + 1; i < end - 1; ++i) {
    const distSq = distanceToSegmentSq(points[i], s, e);
    if (distSq > maxDistSq) {
      maxDistSq = distSq;
      maxNdx = i;
    }
  }
  if (Math.sqrt(maxDistSq) > epsilon) {
    simplifyPoints(points, start, maxNdx + 1, epsilon, outPoints);
    simplifyPoints(points, maxNdx, end, epsilon, outPoints);
  } else {
    if (!outPoints.length) {
      outPoints.push(s);
    }
    outPoints.push(e);
  }
  return outPoints;
}
__name(simplifyPoints, "simplifyPoints");
function pointsOnBezierCurves(points, tolerance = 0.15, distance2) {
  const newPoints = [];
  const numSegments = (points.length - 1) / 3;
  for (let i = 0; i < numSegments; i++) {
    const offset = i * 3;
    getPointsOnBezierCurveWithSplitting(points, offset, tolerance, newPoints);
  }
  if (distance2 && distance2 > 0) {
    return simplifyPoints(newPoints, 0, newPoints.length, distance2);
  }
  return newPoints;
}
__name(pointsOnBezierCurves, "pointsOnBezierCurves");
const appendNewPoint = /* @__PURE__ */ __name((list, point) => {
  const lastPoint = list[list.length - 1];
  if (!lastPoint || lastPoint[0] !== point[0] || lastPoint[1] !== point[1]) {
    list.push(point);
  }
}, "appendNewPoint");
const appendNewPoints = /* @__PURE__ */ __name((list, ...points) => {
  points.forEach((point) => {
    appendNewPoint(list, point);
  });
}, "appendNewPoints");
const gcodeToPoints = /* @__PURE__ */ __name((gcode) => {
  const parsedGcode = gcodetogeometry.parse(gcode.replace(/^%.*/gm, "").replace(/^.*\[.*/gm, ""));
  return parsedGcode.lines.reduce((points, line) => {
    if (line.type === "G1") {
      appendNewPoints(points, [line.start.x, line.start.y], [line.end.x, line.end.y]);
    } else if (line.type === "G2" || line.type === "G3") {
      line.beziers.forEach((point) => {
        if (point.p1.x && point.p1.y && point.p2.x && point.p2.y) {
          const curvePoints = pointsOnBezierCurves([
            [point.p0.x, point.p0.y],
            [point.p1.x, point.p1.y],
            [point.p2.x, point.p2.y],
            [point.p3.x, point.p3.y]
          ], 1e-3, 0.01);
          appendNewPoints(points, ...curvePoints);
        } else {
          appendNewPoints(points, [point.p0.x, point.p0.y], [point.p3.x, point.p3.y]);
        }
      });
    }
    return points;
  }, []);
}, "gcodeToPoints");
const withModal = /* @__PURE__ */ __name((gcode, feedbackUnits) => {
  const returnUnits = feedbackUnits === "mm" ? "G21" : "G20";
  return `
%DISTANCE=modal.distance
%UNITS=modal.units
%START_X=posx
%START_Y=posy
G0 G20 G91 Z0.2
G90
${gcode}
G0 ${returnUnits} G90 X[START_X] Y[START_Y]
G0 G20 G91 Z-0.2
[DISTANCE][UNITS]
`.trim();
}, "withModal");
const fixedDigits = /* @__PURE__ */ __name((number, limit = 4) => number.toFixed(limit).replace(/\.?0+$/, ""), "fixedDigits");
const pointsToGcode = /* @__PURE__ */ __name((points, feedbackUnits) => {
  const pointGcode = points.map(([x, y]) => `G0 X${fixedDigits(x)} Y${fixedDigits(y)}`);
  const sleep = "G04 P.5";
  pointGcode.splice(1, 0, sleep);
  pointGcode.push(sleep);
  return withModal(pointGcode.join("\n"), feedbackUnits);
}, "pointsToGcode");
const orderByClosest = /* @__PURE__ */ __name((points, x, y) => {
  const closestIndex = points.reduce((closest, [px, py], index2) => {
    const distance2 = (x - px) ** 2 + (y - py) ** 2;
    if (distance2 < closest.distance) {
      closest.index = index2;
      closest.distance = distance2;
    }
    return closest;
  }, {
    index: 0,
    distance: Infinity
  });
  const index = closestIndex.index;
  if (index > 0) {
    return [...points.slice(index), ...points.slice(0, index)];
  }
  return points;
}, "orderByClosest");
const returnToEnd = /* @__PURE__ */ __name((points) => [...points, points[0]], "returnToEnd");
const buildOutline = /* @__PURE__ */ __name((gcode, x, y, feedbackUnits = "mm") => {
  const points = gcodeToPoints(gcode);
  const impMultiplier = feedbackUnits === "mm" ? 25.4 : 1;
  const impX = x / impMultiplier;
  const impY = y / impMultiplier;
  const hullPoints = hull__default.default(points);
  const path = returnToEnd(orderByClosest(hullPoints, impX, impY));
  return pointsToGcode(path, feedbackUnits);
}, "buildOutline");
const parentPort = require("worker_threads").parentPort;
parentPort.on("message", (event) => {
  let perimeter = buildOutline(event.gcode, event.x, event.y, event.feedbackUnits);
  parentPort.postMessage({
    data: {
      name: event.name,
      perimeter
    }
  });
});
